/*@java
 @Created by Dmitriy I. Suyasov
 @Date 18.05.2005
 @Time 15:05:12
 */
package dsuyasov.inetDoc.automat;

import dsuyasov.inetDoc.functionality.Rule;
import dsuyasov.inetDoc.functionality.RuleElement;

import java.util.ArrayList;


public class AutomatRuleProcessor extends Automat {

    /*@
         .
         @param automatNumber  
         @param rule  
     */
    public AutomatRuleProcessor(int automatNumber, Rule rule) {
        super(automatNumber);
        this.rule = rule;
        initialize();
    }

    /*@
          .
     */
    protected void initialize() {
        z0_initialization();
    }

    /*@
          .
         @param event   
     */
    public synchronized void automat(int event) {
        int y_statePrevious = y_state;
        log.automatWork(" "+automatNumber+"     " + y_state, true);

        switch (y_state) {
            case (AutomatRuleProcessorConstant.STATE_0_NOT_BEGIN):
                if (AutomatRuleProcessorConstant.EVENT_0_NEW_SYMBOL == event) {
                    if (x0_checkRules()) {
                        y_state = AutomatRuleProcessorConstant.STATE_1_BEGIN;
                        log.automatChangeState(" "+automatNumber+"    0   1:   ");
                    } else {
                        z1_saveSymbol();
                    }
                } else
                if (AutomatRuleProcessorConstant.EVENT_1_NEED_TO_CLOSED == event) {
                    z0_initialization();
                }
                break;
            case (AutomatRuleProcessorConstant.STATE_1_BEGIN):
                if (AutomatRuleProcessorConstant.EVENT_0_NEW_SYMBOL == event) {
                    if (!x1_childInStateOne() && x0_checkRules()) {
                        y_state = AutomatRuleProcessorConstant.STATE_2_END;
                        log.automatChangeState(" "+automatNumber+"    1   2:   ");
                        z2_saveResult();
                    } else {
                        z1_saveSymbol();
                    }
                } else
                if (AutomatRuleProcessorConstant.EVENT_1_NEED_TO_CLOSED == event) {
                    y_state = AutomatRuleProcessorConstant.STATE_0_NOT_BEGIN;
                    log.automatChangeState(" "+automatNumber+"    1   0:   ");
                    z2_saveResult();
                }
                break;
            case (AutomatRuleProcessorConstant.STATE_2_END):
                if (AutomatRuleProcessorConstant.EVENT_0_NEW_SYMBOL == event) {
                    if (x0_checkRules()) {
                        y_state = AutomatRuleProcessorConstant.STATE_1_BEGIN;
                        log.automatChangeState(" "+automatNumber+"    2   1:   ");
                    } else {
                        z1_saveSymbol();
                    }
                } else
                if (AutomatRuleProcessorConstant.EVENT_1_NEED_TO_CLOSED == event) {
                    y_state = AutomatRuleProcessorConstant.STATE_0_NOT_BEGIN;
                    log.automatChangeState(" "+automatNumber+"    2   0:   ");
                }
                break;
        }

        if (y_state != y_statePrevious)
            switch (y_state) {
                case (AutomatRuleProcessorConstant.STATE_0_NOT_BEGIN):
                    z0_initialization();
                    break;
                case (AutomatRuleProcessorConstant.STATE_1_BEGIN):
                    z3_initializeEndSearching();
                    break;
                case (AutomatRuleProcessorConstant.STATE_2_END):
                    z0_initialization();
                    break;
            }

        log.automatWork(" "+automatNumber+"     " + y_state, false);
    }

    //  

    /*@
               
         @return      
     */
    private boolean x0_checkRules() {
        String text = stack + newSymbol;

        ArrayList list = new ArrayList();
        ArrayList nextRules = new ArrayList();
        list.add(new Integer(text.length()));
        nextRules.add(0, new Object[] {new Integer(0), null});

        for (int iLoop = rules.size() - 1; iLoop >=0; iLoop--) {
            RuleElement ruleElement = (RuleElement) rules.get(iLoop);
            ruleElement.clearNextList();
            int type = ruleElement.getType();
            ArrayList list2 = new ArrayList();
            int index = 0;
            switch (type) {
                case RuleElement.TYPE_CAN_ONE:
                    for (int jLoop = 0; jLoop < list.size(); jLoop++) {
                        list2.addAll(ruleElement.satisfy(text.substring(0, ((Integer)list.get(jLoop)).intValue()), (Integer) ((Object[]) nextRules.get(jLoop))[0], (RuleElement) ((Object[]) nextRules.get(jLoop))[1]));
                    }
                    list.addAll(list2);
                    for (int i = 0; i < list2.size(); i++) nextRules.add(new Object[] {new Integer(i), ruleElement});
                    break;
                case RuleElement.TYPE_ONE:
                    for (int jLoop = 0; jLoop < list.size(); jLoop++) {
                        list2.addAll(ruleElement.satisfy(text.substring(0, ((Integer)list.get(jLoop)).intValue()), (Integer) ((Object[]) nextRules.get(jLoop))[0], (RuleElement) ((Object[]) nextRules.get(jLoop))[1]));
                    }
                    list = list2;
                    nextRules.clear();
                    for (int i = 0; i < list2.size(); i++) nextRules.add(new Object[] {new Integer(i), ruleElement});
                    if (list.size() == 0) {
                        log.inputVariable("x0 -  ", false);
                        return false;
                    }
                    break;
                case RuleElement.TYPE_ONE_OR_MORE:
                    for (int jLoop = 0; jLoop < list.size(); jLoop++) {
                        list2.addAll(ruleElement.satisfy(text.substring(0, ((Integer)list.get(jLoop)).intValue()), (Integer) ((Object[]) nextRules.get(jLoop))[0], (RuleElement) ((Object[]) nextRules.get(jLoop))[1]));
                    }
                    if (list2.size() == 0) {
                        log.inputVariable("x0 -  ", false);
                        return false;
                    }
                    nextRules.clear();
                    for (int i = 0; i < list2.size(); i++) nextRules.add(new Object[] {new Integer(i), ruleElement});
                    index = list.size();
                    list = list2;
                case RuleElement.TYPE_ZERO_OR_MORE:
                    list2.clear();
                    for (int jLoop = 0; jLoop < list.size(); jLoop++) {
                        list2.addAll(ruleElement.satisfy(text.substring(0, ((Integer)list.get(jLoop)).intValue()),  (Integer) ((Object[]) nextRules.get(index+jLoop))[0], (RuleElement) ((Object[]) nextRules.get(jLoop))[1]));
                    }
                    list.addAll(list2);
                    for (int i = 0; i < list2.size(); i++) nextRules.add(new Object[] {new Integer(index+i), ruleElement});

                    while (list2.size() != 0) {
                        index += list2.size();
                        ArrayList list3 = new ArrayList();
                        for (int jLoop = 0; jLoop < list2.size(); jLoop++) {
                            list3.addAll(ruleElement.satisfy(text.substring(0, ((Integer)list2.get(jLoop)).intValue()), new Integer(index+jLoop), ruleElement));
                        }
                        list2 = list3;
                        list.addAll(list2);
                        for (int i = 0; i < list2.size(); i++) nextRules.add(new Object[] {new Integer(index+i), ruleElement});
                    }
                    break;
            }
        }
        int position = 0;
        if (y_state == AutomatRuleProcessorConstant.STATE_1_BEGIN)
            position = rule.getEndPosition();
        else
            position = rule.getStartPosition();

        RuleElement ruleElement = (RuleElement) rules.get(0);

        RuleElement tRuleElement = null;

        int link = 0;
        int index = 0;
        int next = 0;
        while ((position != index) && (ruleElement != null)) {
            next = ruleElement.lastSuccessed(link);
            tRuleElement = ruleElement.lastSuccessedLink(link);
            for (int i = 0; i < rules.size(); i++)
                if (tRuleElement == (RuleElement) rules.get(i))
                    index = i;
            ruleElement = tRuleElement;
            link = next;
        }
        if (position == index) {
            if (ruleElement == null)
                stackCoincidencePosition = 0;
            else
                stackCoincidencePosition = text.length() - ruleElement.position(link);
        } else {
                stackCoincidencePosition = 0;
        }
        log.inputVariable("x0 -  ", true);
        return true;
    }

    /*@
         ,      
     */
    private boolean x1_childInStateOne() {
        ArrayList children = rule.getAllRules();
        for (int i = 0; i < children.size(); i++)
            if (((Rule) children.get(i)).automatState() == AutomatRuleProcessorConstant.STATE_1_BEGIN) {
                log.inputVariable("x1 -     ", true);
                return true;
            }
        log.inputVariable("x1 -     ", false);
        return false;
    }

    //  

    /*@
            
     */
    private void z0_initialization() {
        log.output("z0 -    ");
        rules = rule.getStartRules();
        this.stack = "";
    }

    /*@
           
     */
    private void z1_saveSymbol() {
        log.output("z1 -   ");
        stack += newSymbol;
    }

    /*@
           
     */
    private void z2_saveResult() {
        log.output("z2 -       ");
        ruleResult += (stack+newSymbol).substring(0, (stack+newSymbol).length() - stackCoincidencePosition);

        ArrayList replaces = rule.getReplaces();
        for (int iLoop = 0; iLoop < replaces.size(); iLoop++) {
            String from = ((String[])(replaces.get(iLoop)))[0];
            String to = ((String[])(replaces.get(iLoop)))[1];
            int res = ruleResult.indexOf(from);
            int index = 0;
            while ((index<ruleResult.length()) && (res>=0)) {
                index = (((res>0)?ruleResult.substring(0,res):"") + to).length();
                ruleResult = ((res>0)?ruleResult.substring(0,res):"") + to + ((res+from.length()<ruleResult.length())?ruleResult.substring(res+from.length()):"");
                res = ruleResult.indexOf(from, index);
            }
        }
    }

    /*@
            
     */
    private void z3_initializeEndSearching() {
        log.output("z3 -    ");
        rules = rule.getEndRules();
        this.stack = (stack+newSymbol).substring((stack+newSymbol).length() - stackCoincidencePosition);
        ruleResult = "";
    }

    /*@
          
     */
    private Rule rule;
    /*@
          / 
     */
    private ArrayList rules;

    /*@
           
     */
    public String stack;
    /*@
          
     */
    public String ruleResult = "";
    /*@
            /   
     */
    public int stackCoincidencePosition = 0;
    /*@
          
     */
    public char newSymbol = ' ';

    /*@
           
         @return   
     */
    public int getState() {
        return y_state;
    }
}