/*@java
 @Created by Dmitriy I. Suyasov
 @Date 18.05.2005
 @Time 11:32:48
 */
package dsuyasov.inetDoc.functionality;

import org.w3c.dom.Node;

import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;

import dsuyasov.inetDoc.automat.AutomatRuleProcessor;
import dsuyasov.inetDoc.automat.AutomatRuleProcessorConstant;

public class Rule {

    /*@
          -  
    */
    public static final String NODE_START = "start";
    /*@
          -  
    */
    public static final String NODE_END = "end";
    /*@
          - 
    */
    public static final String NODE_DEF = "def";
    /*@
          - 
    */
    public static final String NODE_MUST = "relations";
    /*@
          -   
    */
    public static final String NODE_SEPARATOR = "separator";
    /*@
          - 
    */
    public static final String NODE_RULE = "rule";

    /*@
             
    */
    public static final String SEPARATOR = "|";

    /*@
         
    */
    private String name;
    /*@
         
    */
    private Map definitions;
    /*@
           
    */
    private ArrayList startRules;
    /*@
           
    */
    private ArrayList endRules;
    /*@
          
    */
    private String separator = null;
    /*@
          
    */
    private ArrayList rules;
    /*@
         
    */
    private ArrayList replaces;

    /*@
              
    */
    private ArrayList unresolvedLinks;
    /*@
         ,    
    */
    private ArrayList linked;
    /*@
         ,    
    */
    private ArrayList links;
    /*@
         -   
    */
    private AutomatRuleProcessor ruleProcessor = null;
    /*@
         - 
    */
    private Rule parent;

    /*@
                 
    */
    private int startPosition  = -1;
    /*@
                 
    */
    private int endPosition = -1;

    /*@
     .    XML .
     @param xml    XML 
     @param node    
     @param defines    -
     @param parent   -
     */
    public Rule(XMLProcessor xml, Node node, Map defines, Rule parent) throws Exception {
        linked = new ArrayList();
        links = new ArrayList();
        this.parent = parent;
        try {
            name = node.getAttributes().getNamedItem("name").getNodeValue();
            definitions = new HashMap(defines);
            replaces = new ArrayList(xml.getReplaces(node));
            startRules = new ArrayList();
            endRules = new ArrayList();
            unresolvedLinks = new ArrayList();

            definitions.putAll(xml.getDefs(node));
            // Find inside rules
            rules = new ArrayList();
            ArrayList ruleNodes = xml.getRules(node);
            for (int iLoop = 0; iLoop < ruleNodes.size(); iLoop++)
                rules.add(new Rule(xml, (Node) ruleNodes.get(iLoop), definitions, this));
            for (int iLoop = 0; iLoop < rules.size(); iLoop++)
                ((Rule)rules.get(iLoop)).realizeLinks(rules);

            // Get separator if exist
            Node separatorNode = XMLProcessor.getNode(node, NODE_SEPARATOR, 1);
            separator = (separatorNode == null)?null:XMLProcessor.getNodeValue(separatorNode);

            // Get rule start
            Node startNode = XMLProcessor.getNode(node, NODE_START, 1);
            if (startNode == null)
                throw new Exception("Tag \"rule\" should contain tag \"start\".");
            String value = XMLProcessor.getNodeValue(startNode);
            int index = value.indexOf(SEPARATOR);
            while (index >= 0) {
                String tmp = value.substring(0, index);
                if (tmp.charAt(0) == 'P') {
                    startPosition = startRules.size();
                    tmp = tmp.substring(1);
                }
                startRules.add(new RuleElement(tmp, definitions));
                value = value.substring(index+1);
                index = value.indexOf(SEPARATOR);
            }
            if (value.charAt(0) == 'P') {
                startPosition = startRules.size();
                value = value.substring(1);
            }
            startRules.add(new RuleElement(value, definitions));
            if (startPosition < 0) startPosition = startRules.size();

            // Get rule end
            Node endNode = XMLProcessor.getNode(node, NODE_END, 1);
            if (endNode == null)
                throw new Exception("Tag \"rule\" should contain tag \"start\".");
            value = XMLProcessor.getNodeValue(endNode);
            index = value.indexOf(SEPARATOR);
            while (index >= 0) {
                String tmp = value.substring(0, index);
                if (tmp.charAt(0) == 'P') {
                    endPosition = endRules.size()+1;
                    tmp = tmp.substring(1);
                }
                endRules.add(new RuleElement(tmp, definitions));
                value = value.substring(index+1);
                index = value.indexOf(SEPARATOR);
            }
            if (value.charAt(0) == 'P') {
                endPosition = endRules.size()+1;
                value = value.substring(1);
            }
            endRules.add(new RuleElement(value, definitions));
            if (endPosition < 0) endPosition = 0;

            // Get rule relations
            index = 1;
            Node relationNode = XMLProcessor.getNode(node, NODE_MUST, index);
            while (relationNode != null) {
                String linkName = relationNode.getAttributes().getNamedItem("name").getNodeValue();
                String linkType = relationNode.getAttributes().getNamedItem("type").getNodeValue();
                if ((linkName==null) || (linkType==null) || (linkName.length()<=0) || (linkType.length()<=0))
                    throw new Exception("\"Must\" tag should contain not empty \"Name\" and \"Type\" attributes!");
                if (linkName.equals(name))
                    throw new Exception("\"Name\" attribute of \"Must\" tag should differ from rule name!");
                unresolvedLinks.add(new String[] {linkName, linkType});
                index++;
                relationNode = XMLProcessor.getNode(node, NODE_MUST, index);
            }
            ruleProcessor = new AutomatRuleProcessor(10, this);
        } catch (Exception e) {
            throw new Exception("Incorrect xml file. Error found: "+e.getMessage());
        }
    }

    /*@
           ,   
        @param rules   
    */
    public void realizeLinks(ArrayList rules) throws Exception{
        Iterator iter = unresolvedLinks.iterator();
        while (iter.hasNext()) {
            String[] unresolvedLink = (String[]) iter.next();
            String lname = unresolvedLink[0];
            String ltype = unresolvedLink[1];
            Iterator iter2 = rules.iterator();
            while (iter2.hasNext()) {
                Rule tRule = (Rule) iter2.next();
                if (tRule.getName().equals(lname)) {
                    tRule.setLinked(this);
                    links.add(new RuleLink(ltype, tRule));
                    break;
                }
            }
        }
    }

    /*@
             
        @param name  
        @return  
    */
    public ArrayList getRule(String name) {
        ArrayList toReturn = new ArrayList();
        if (this.name.equals(name))
            toReturn.add(this);
        for (int i = 0; i < rules.size(); i++)
            toReturn.addAll(((Rule)rules.get(i)).getRule(name));
        return toReturn;
    }

    /*@
           
        @param map   
    */
    public void addDifinitions(Map map) {
        definitions.putAll(map);
    }

    /*@
          
        @return  
    */
    public String getName() {
        return name;
    }

    /*@
         ,     
        @param link    
    */
    public void setLinked(Rule link) {
        linked.add(link);
    }

    /*@
           
        @return    
    */
    public ArrayList getStartRules() {
        return startRules;
    }

    /*@
           
        @return    
    */
    public ArrayList getEndRules() {
        return endRules;
    }

    /*@
          
        @return  
    */
    public Map getDefinitions() {
        return definitions;
    }

    /*@
          ,   .
        @return  
    */
    public ArrayList getLinked() {
        return linked;
    }

    /*@
          ,       .
        @return  
    */
    public ArrayList getLinks() {
        return links;
    }

    /*@
         -
        @return 
    */
    public Rule getParent() {
        return this.parent;
    }

    /*@
          
        @return  
    */
    public ArrayList getAllRules() {
        return rules;
    }

    /*@
           
        @return  
    */
    public String getRuleResult() {
        return ruleProcessor.ruleResult;
    }

    /*@
          - 
        @return   
    */
    public int automatState() {
        return ruleProcessor.getState();
    }

    /*@
            
        @return  
    */
    public String getSeparator() {
        return this.separator;
    }

    /*@
          (  )   
        @param symbol 
    */
    public void newSymbolCame(char symbol) {
        this.ruleProcessor.newSymbol = symbol;
        this.ruleProcessor.automat(AutomatRuleProcessorConstant.EVENT_0_NEW_SYMBOL);
    }

    /*@
          (  )   
        @param backSteps  ,        
    */
    public void parentRuleClosed(int backSteps) {
        this.ruleProcessor.stackCoincidencePosition = backSteps;
        this.ruleProcessor.automat(AutomatRuleProcessorConstant.EVENT_1_NEED_TO_CLOSED);
    }

    /*@
          (  )   
    */
    public void setEventNeedToCloseForChildren() {
        parentRuleClosed(0);
        for (int i = 0; i < rules.size(); i++) {
            ((Rule) rules.get(i)).parentRuleClosed(0);
            ((Rule) rules.get(i)).setEventNeedToCloseForChildren();
        }
    }

    /*@
             /   
         @return  
     */
    public int getBackSteps() {
        return this.ruleProcessor.stackCoincidencePosition;
    }

    /*@
             
         @return  
     */
    public int getStartPosition() {
        return startPosition;
    }
    /*@
             
         @return  
     */
    public int getEndPosition() {
        return endPosition;
    }

    /*@
           - 
         @return  
     */
    public String getStack() {
        return ruleProcessor.stack;
    }
    /*@
           - 
         @param stack  
     */
    public void setStack(String stack) {
        ruleProcessor.stack = stack;
    }

    /*@
           
         @return  
     */
    public ArrayList getReplaces() {
        return replaces;
    }

}
