/*@java
 @Created by Dmitriy I. Suyasov
 @Date 07.04.2005
 @Time 13:57:13
 */
package dsuyasov.inetDoc.functionality;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.util.ArrayList;
import java.util.Map;
import java.util.Iterator;


public class Template {

    /*@
             
     */
    public static final int BASE_ENTITY = 0;
    /*@
            
     */
    public static final int HEAD_ENTITY = 1;
    /*@
            
     */
    public static final int DEFAULT_ENTITY = 2;

    /*@
           ,     
     */
    public ArrayList types;
    /*@
          
     */
    public ArrayList entities;

    /*@
            
     */
    public ArrayList rules;

    /*@
         .
         @param types   ,     
         @param templateFileName    
     */
    public Template(ArrayList types, String templateFileName) throws Exception {

        this.types = types;

        DocumentBuilder db =  DocumentBuilderFactory.newInstance().newDocumentBuilder();
        Document doc = db.parse(templateFileName);

        XMLProcessor xml = new XMLProcessor(doc);

        try {
            entities = new ArrayList();
            entities.add(BASE_ENTITY, new Entity(xml, xml.getBaseEntity()));
            entities.add(HEAD_ENTITY, new Entity(xml, xml.getHeadEntity()));
            String defaultEntity = xml.getDefaultEntity();
            Node[] entityNodes = xml.getEntities();
            for (int iLoop = 0; iLoop < entityNodes.length; iLoop++) {
                Entity entity = new Entity(xml, entityNodes[iLoop]);
                if (entity.entityName.equals(defaultEntity))
                    entities.add(DEFAULT_ENTITY, entity);
                else
                    entities.add(entity);
            }
            Node ruleNode = xml.getRuleNode(doc.getElementsByTagName("template").item(0));
            ArrayList ruleNodes = xml.getRules(ruleNode);
            Map defMap = xml.getDefs(ruleNode);
            rules = new ArrayList();
            try {
                for (int iLoop = 0; iLoop < ruleNodes.size(); iLoop++)
                    rules.add(new Rule(xml, (Node) ruleNodes.get(iLoop), defMap, null));
                for (int iLoop = 0; iLoop < rules.size(); iLoop++)
                    ((Rule)rules.get(iLoop)).realizeLinks(rules);
            } catch (Exception e) {
                throw new XMLProcessorException(e.getMessage());
            }

            Iterator eIter = entities.iterator();
            while (eIter.hasNext()) {
                String entityName = ((Entity) eIter.next()).entityName;
                if (entityName.equals(""))
                    continue;
                ArrayList entitiesRule = getRules(entityName);
                if (entitiesRule.size() != 1)
                    throw new XMLProcessorException("Base rules should contain one at a time rule for every entity.");
            }
        } catch (XMLProcessorException xmlpe) {
            throw new Exception(xmlpe.getMessage());
        }

    }

    /*@
              
         @param name  
         @return  
     */
    public ArrayList getRules(String name) {
        Iterator iter = rules.iterator();
        ArrayList entitiesRule = new ArrayList();
        while (iter.hasNext())
            entitiesRule.addAll(((Rule) iter.next()).getRule(name));
        return entitiesRule;
    }

    /*@
               
         @param name  
         @return 
     */
    public Rule getFirstRule(String name) {
        Iterator iter = rules.iterator();
        ArrayList entitiesRule = new ArrayList();
        while (iter.hasNext() && (entitiesRule.size()==0))
            entitiesRule.addAll(((Rule) iter.next()).getRule(name));
        if (entitiesRule.size()==0)
            return null;
        return (Rule) entitiesRule.get(0);
    }

    /*@
             
         @param name  
         @return 
     */
    public Entity getEntity(String name) {
        for (int iLoop = 0; iLoop < entities.size(); iLoop++)
            if (((Entity)entities.get(iLoop)).entityName.equals(name))
                return (Entity) entities.get(iLoop);
        return (Entity) entities.get(Template.DEFAULT_ENTITY);
    }

    public void initiateRules() {
        for (int i = 0; i < rules.size(); i++)
            ((Rule) rules.get(i)).setEventNeedToCloseForChildren();
        for (int j = 0; j < entities.size(); j++)
            if (((Entity)entities.get(j)).entityName != "")
                for (int i = 0; i < ((Entity)entities.get(j)).rules.size(); i++)
                    ((Rule) ((Entity)entities.get(j)).rules.get(i)).setEventNeedToCloseForChildren();
    }

}