/*@java
 @Created by Dmitriy I. Suyasov
 @Date 08.04.2005
 @Time 11:58:19
 */
package dsuyasov.inetDoc.functionality;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

public class XMLProcessor {

    /*@
        XML 
    */
    public Document xml;

    /*@
        .
        @param doc XML 
    */
    public XMLProcessor(Document doc) {
        xml = doc;
    }

    /*@
          XML ,    
        @return  XML
    */
    public Node getBaseEntity() throws XMLProcessorException {
        NodeList nodeList = xml.getElementsByTagName("baseentity");
        if ((nodeList == null) || (nodeList.getLength() != 1))
            throw new XMLProcessorException("Template has " + nodeList.getLength() + " base entities.");
        return nodeList.item(0);
    }
    /*@
          XML ,    
        @return  XML
    */
    public Node getHeadEntity() throws XMLProcessorException {
        NodeList nodeList = xml.getElementsByTagName("headentity");
        if ((nodeList == null) || (nodeList.getLength() != 1))
            throw new XMLProcessorException("Template has " + nodeList.getLength() + " head entities.");
        return nodeList.item(0);
    }
    /*@
             XML 
        @param node  XML 
        @return  
    */
    public ArrayList getEntitiesAtributes(Node node) throws XMLProcessorException {
        ArrayList list = new ArrayList();
        Node defaultAttribute = null;
        for (int iLoop = 0; iLoop < node.getChildNodes().getLength(); iLoop++) {
            Node child = node.getChildNodes().item(iLoop);
            if (child.getNodeName().equals("defaultattribute"))
                defaultAttribute = child;
        }
        if ((defaultAttribute == null) || (defaultAttribute.getAttributes().getNamedItem("name") == null))
            throw new XMLProcessorException("Node has no child named \"defaultattribute\".");
        for (int iLoop = 0; iLoop < node.getChildNodes().getLength(); iLoop++) {
            Node child = node.getChildNodes().item(iLoop);
            if ((child.getNodeName().equals("attribute")) && (child.getAttributes().getNamedItem("name") != null))
                if (child.getAttributes().getNamedItem("name").getNodeValue().equals(defaultAttribute.getAttributes().getNamedItem("name").getNodeValue()))
                    list.add(0, child.getAttributes().getNamedItem("name").getNodeValue());
                else
                    list.add(child.getAttributes().getNamedItem("name").getNodeValue());
        }
        if (list.size() < 1)
            throw new XMLProcessorException("Node hasn't any attribute childs.");
        return list;
    }

    /*@
           
        @param node  XML 
        @return  
    */
    public String getTemplateName(Node node) throws XMLProcessorException {
        for (int iLoop = 0; iLoop < node.getChildNodes().getLength(); iLoop++) {
            Node child = node.getChildNodes().item(iLoop);
            if (child.getNodeName().equals("template-file") && (child.getAttributes().getNamedItem("href") != null))
                return child.getAttributes().getNamedItem("href").getNodeValue();
        }
        throw new XMLProcessorException("Node has no template file node");
    }

    /*@
            
        @return  
    */
    public String getDefaultEntity() throws XMLProcessorException {
        NodeList nodeList = xml.getElementsByTagName("defaultentity");
        if ((nodeList == null) || (nodeList.getLength() != 1))
            throw new XMLProcessorException("Xml file has " + nodeList.getLength() + " nodes \"defaultentity\"");
        Node node = nodeList.item(0);
        if (node.getAttributes().getNamedItem("name") == null)
            throw new XMLProcessorException("Xml file has no nodes \"defaultentity\" with attribute \"name\"");
        return node.getAttributes().getNamedItem("name").getNodeValue();
    }

    /*@
             XML 
        @return  
    */
    public Node[] getEntities() throws XMLProcessorException {
        NodeList nodeList = xml.getElementsByTagName("entity");
        if ((nodeList == null) || (nodeList.getLength() < 1))
            throw new XMLProcessorException("Xml document hasn't default entity");
        Node[] nodes = new Node[nodeList.getLength()];
        for (int iLoop = 0; iLoop < nodeList.getLength(); iLoop++)
            nodes[iLoop] = nodeList.item(iLoop);
        return nodes;
    }

    /*@
          
        @param node  XML 
        @return  
    */
    public String getEntityName(Node node) throws XMLProcessorException {
        if (node.getAttributes().getNamedItem("name") == null)
            throw new XMLProcessorException("Entity hasn't any name.");
        return node.getAttributes().getNamedItem("name").getNodeValue();
    }
    /*@
            
        @param node  XML 
        @return    
    */
    public Node getRuleNode(Node node) throws XMLProcessorException {
        for (int i = 0; i < node.getChildNodes().getLength(); i++)
            if (node.getChildNodes().item(i).getNodeName().equals("rules"))
                return node.getChildNodes().item(i);
        throw new XMLProcessorException("Node \"rules\" wasn't found.");
    }
    /*@
            
        @param node  XML 
        @return   
    */
    public ArrayList getRules(Node node) {
        ArrayList toReturn = new ArrayList();
        for (int j = 0; j < node.getChildNodes().getLength(); j++) {
            if (node.getChildNodes().item(j).getNodeName().equals("rule"))
                toReturn.add(node.getChildNodes().item(j));
        }
        return toReturn;
    }
    /*@
          ,   XML 
        @param node  XML 
        @return  
    */
    public ArrayList getReplaces(Node node) throws XMLProcessorException {
        ArrayList toReturn = new ArrayList();
        for (int j = 0; j < node.getChildNodes().getLength(); j++)
            if (node.getChildNodes().item(j).getNodeName().equals("replace")) {
                Node n = node.getChildNodes().item(j);
                String name = "";
                String value = "";
                for (int k = 0; k < n.getChildNodes().getLength(); k++)
                    if (n.getChildNodes().item(k).getNodeType() == Node.ELEMENT_NODE) {
                        String tName = n.getChildNodes().item(k).getNodeName();
                        if (tName.equals("from")) {
                            name = XMLProcessor.getNodeValue(n.getChildNodes().item(k));
                        } else if (tName.equals("to")) {
                            value = XMLProcessor.getNodeValue(n.getChildNodes().item(k));
                        } else throw new XMLProcessorException("Unknown tag in node \"replace\".");
                    }
                if ((name == null) || (name == ""))
                    throw new XMLProcessorException("Definitions has no \"old\" attribute.");
                if ((value == null) || (value == ""))
                    throw new XMLProcessorException("Definitions has no \"new\" attribute.");
                int index=0;
                while (index<value.length()) {
                    index = value.indexOf("\\", index);
                    if (index<0) index = value.length();
                    if (index+1<value.length()) {
                        String ch = "";
                        switch (value.charAt(index+1)) {
                            case '\\':
                                ch = "\\";
                                break;
                            case 'n':
                                ch = "\n";
                                break;
                            case 'r':
                                ch = "\r";
                                break;
                            case 't':
                                ch = "\t";
                                break;
                        }
                        value = ((index>0)?value.substring(0, index):"") + ch + ((index+2<value.length())?value.substring(index+2):"");
                        index+=1;
                    }
                }
                index=0;
                while (index<name.length()) {
                    index = name.indexOf("\\", index);
                    if (index<0) index = name.length();
                    if (index+1<name.length()) {
                        String ch = "";
                        switch (name.charAt(index+1)) {
                            case '\\':
                                ch = "\\";
                                break;
                            case 'n':
                                ch = "\n";
                                break;
                            case 'r':
                                ch = "\r";
                                break;
                            case 't':
                                ch = "\t";
                                break;
                        }
                        name = ((index>0)?name.substring(0, index):"") + ch + ((index+2<name.length())?name.substring(index+2):"");
                        index+=1;
                    }
                }
                toReturn.add(new String[] {name, value});
            }
        return toReturn;
    }
    /*@
            
        @param node  XML 
        @return  
    */
    public Map getDefs(Node node)  throws XMLProcessorException {
        Map toReturn = new HashMap();
        for (int j = 0; j < node.getChildNodes().getLength(); j++)
            if (node.getChildNodes().item(j).getNodeName().equals("def")) {
                String name = node.getChildNodes().item(j).getAttributes().getNamedItem("name").getNodeValue();
                if ((name == null) || (name == ""))
                    throw new XMLProcessorException("Definitions has no name attribute.");
                String value = XMLProcessor.getNodeValue(node.getChildNodes().item(j));
                if ((value == null) || (value == ""))
                    throw new XMLProcessorException("Definitions has no value.");
                int index=0;
                while (index<value.length()) {
                    index = value.indexOf("\\", index);
                    if (index<0) index = value.length();
                    if (index+1<value.length()) {
                        String ch = "";
                        switch (value.charAt(index+1)) {
                            case '\\':
                                ch = "\\";
                                break;
                            case 'n':
                                ch = "\n";
                                break;
                            case 'r':
                                ch = "\r";
                                break;
                            case 't':
                                ch = "\t";
                                break;
                        }
                        value = ((index>0)?value.substring(0, index):"") + ch + ((index+2<value.length())?value.substring(index+2):"");
                        index+=1;
                    }
                }
                toReturn.put(name, value);
            }
        return toReturn;
    }
    /*@
               
        @param node  XML 
        @param name  
        @param number  
        @return      
    */
    public static Node getNode(Node node, String name, int number) {
        if (number < 1)
            return null;
        if (node.getNodeType() != Node.ELEMENT_NODE)
            return null;

        for (int iLoop = 0; iLoop < node.getChildNodes().getLength(); iLoop++) {
            if (node.getChildNodes().item(iLoop).getNodeName().equals(name))
                number--;
            if (number == 0)
                return node.getChildNodes().item(iLoop);
        }
        return null;
    }

    /*@
           
        @param node  XML 
        @return 
    */
    public static String getNodeValue(Node node) {
        for (int iLoop = 0; iLoop < node.getChildNodes().getLength(); iLoop++)
            if (node.getChildNodes().item(iLoop).getNodeType() == Node.CDATA_SECTION_NODE) {
                node = node.getChildNodes().item(iLoop);
                break;
            }
        return node.getNodeValue();
    }
}
