/**
 @author Dmitriy I. Suyasov
 @since 05.09.2005 13:17:48
 */
package dsuyasov.inetDoc.automat;

import dsuyasov.inetDoc.xml.*;

import java.io.File;
import java.io.FileReader;
import java.util.Stack;

public class AutomatTemplateParser extends Automat {

    public static final int AUTOMAT_NUMBER = 8;

    public static final int STATE_0_READ_TEMPLATE_FILE = 0;
    public static final int STATE_1_GET_NS_TAG = 1;
    public static final int STATE_2_PROCESS_NS_TAG = 2;
    public static final int STATE_3_PROCESS_ATTRIBUTES = 3;

    private String NS_START_PREFIX = "<";
    private String NS_END_PREFIX = "</";

    private Stack nodeStack;

    private String templateFileName = "";
    private String templateFileText = "";

    public RootDocument document;

    public String ERROR = "";
    private int position;

    private int currentTagPosition = -1;

    public AutomatTemplateParser() {
        super("dsuyasov.inetDoc.automat.AutomatTemplateParser");
    }

    public void startAutomat(String templateFileName, String nameSpace) {
        this.templateFileName = templateFileName;
        if (nameSpace != null) {
            NS_START_PREFIX = "<" + nameSpace + ":";
            NS_END_PREFIX = "</" + nameSpace + ":";
        }
        work();
    }

    /*@
          
    */
    protected void initialize() {
        z0_initialize();
        z1_readTemplateFile();
    }

    /*@
          
        @param event   
    */
    protected void automat(int event) {
        int y_previousState = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_READ_TEMPLATE_FILE:
                if (x0_hasError()) {
                    z10_closeAutomat();
                } else {
                    y_state = STATE_1_GET_NS_TAG;
                    log.automatChangeState(automatName, 0, 1);
                }
                break;
            case STATE_1_GET_NS_TAG:
                if (x0_hasError()) {
                    z10_closeAutomat();
                } else if (x1_templateHasUnprocessedTag()) {
                    if (x2_textBeforeTag()) {
                        z3_createTextTeg();
                        y_state = STATE_2_PROCESS_NS_TAG;
                        log.automatChangeState(automatName, 1, 2);
                    } else {
                        y_state = STATE_2_PROCESS_NS_TAG;
                        log.automatChangeState(automatName, 1, 2);
                    }
                } else if (!x7_positialEqualWithTextEnd()) {
                    z9_saveLastText();
                } else if (x8_stackHasMoreThenOneElement()) {
                    z4_setErrorAsSintaxError();
                    z10_closeAutomat();
                } else {
                    z10_closeAutomat();
                }
                break;
            case STATE_2_PROCESS_NS_TAG:
                if (x3_tagIsOpen()) {
                    y_state = STATE_3_PROCESS_ATTRIBUTES;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x4_tagNameEqualToStackTopTagName()) {
                    z5_closeStackTopNode();
                    z6_setPositionAfterTagClosing();
                    y_state = STATE_1_GET_NS_TAG;
                    log.automatChangeState(automatName, 2, 1);
                } else {
                    z4_setErrorAsSintaxError();
                    z10_closeAutomat();
                }
                break;
            case STATE_3_PROCESS_ATTRIBUTES:
                if (x0_hasError()) {
                    z10_closeAutomat();
                } else if (x5_tagHasUnresolvedAttribute()) {
                    z8_saveNodeAttribute();
                } else if (x6_tagHasNoChildren()) {
                    z5_closeStackTopNode();
                    z6_setPositionAfterTagClosing();
                    y_state = STATE_1_GET_NS_TAG;
                    log.automatChangeState(automatName, 3, 1);
                } else {
                    z6_setPositionAfterTagClosing();
                    y_state = STATE_1_GET_NS_TAG;
                    log.automatChangeState(automatName, 3, 1);
                }
                break;
        }

        if (y_previousState != y_state)
            switch (y_state) {
                case STATE_1_GET_NS_TAG:
                    z2_getTagPosition();
                    break;
                case STATE_3_PROCESS_ATTRIBUTES:
                    z7_saveNewElementNode();
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }


    //  

    private boolean x0_hasError() {
        boolean result = ERROR.length() > 0;
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_templateHasUnprocessedTag() {
        boolean result = currentTagPosition > 0;
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_textBeforeTag() {
        boolean result = currentTagPosition > position;
        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_tagIsOpen() {
        boolean result = !templateFileText.substring(currentTagPosition, currentTagPosition + 2).equals("</");
        log.inputVariable(automatName, 3, result);
        return result;
    }

    private boolean x4_tagNameEqualToStackTopTagName() {
        String name = templateFileText.substring(currentTagPosition + NS_END_PREFIX.length(), templateFileText.indexOf(">", currentTagPosition));
        while ("\n\r \t".indexOf(name.charAt(name.length() - 1)) > 0) name = name.substring(0, name.length() - 1);
        Node node = (Node) nodeStack.peek();
        boolean result = node.getName().equals(name);
        log.inputVariable(automatName, 4, result);
        return result;
    }

    private boolean x5_tagHasUnresolvedAttribute() {
        String nodeString = templateFileText.substring(position, templateFileText.indexOf(">", position));
        if ((nodeString.length()>0) && (nodeString.charAt(nodeString.length() - 1) == '/'))
            nodeString = nodeString.substring(0, nodeString.length() - 1);
        int pos = 0;
        while ((pos<nodeString.length()) && Character.isWhitespace(nodeString.charAt(pos))) pos++;
        boolean result = pos < nodeString.length();
        log.inputVariable(automatName, 5, result);
        return result;
    }

    private boolean x6_tagHasNoChildren() {
        boolean result = templateFileText.charAt(templateFileText.indexOf(">", position) - 1) == '/';
        log.inputVariable(automatName, 6, result);
        return result;
    }

    private boolean x7_positialEqualWithTextEnd() {
        boolean result = position >= templateFileText.length();
        log.inputVariable(automatName, 7, result);
        return result;
    }

    private boolean x8_stackHasMoreThenOneElement() {
        boolean result = nodeStack.size() > 1;
        log.inputVariable(automatName, 8, result);
        return result;
    }

    //  

    private void z0_initialize() {
        log.output(automatName, 0);
        templateFileText = "";
        ERROR = "";
        document = new RootDocument();
        position = 0;
        nodeStack = new Stack();
        nodeStack.push(document);
    }

    private void z1_readTemplateFile() {
        log.output(automatName, 1);
        File file = new File(this.templateFileName);
        FileReader fr;
        char charbuff[] = new char [(int) file.length()];
        try {
            fr = new FileReader(file);
            fr.read(charbuff);
            templateFileText = new String(charbuff);
        } catch (Exception e) {
            ERROR = "     ["+templateFileName+"]: " + e.getMessage();
        }
    }

    private void z2_getTagPosition() {
        log.output(automatName, 2);
        int openTag = templateFileText.indexOf(NS_START_PREFIX, position);
        int closeTag = templateFileText.indexOf(NS_END_PREFIX, position);
        if (openTag >= 0)
            if (closeTag >= 0)
                currentTagPosition = (openTag<closeTag)?openTag:closeTag;
            else
                currentTagPosition = openTag;
        else
            currentTagPosition = closeTag;
    }

    private void z3_createTextTeg() {
        log.output(automatName, 3);
        Node node = (Node) nodeStack.pop();
        node.addNode(new TextNode(templateFileText.substring(position, currentTagPosition)));
        nodeStack.push(node);
        position = currentTagPosition + 1;
    }

    private void z4_setErrorAsSintaxError() {
        log.output(automatName, 4);
        ERROR = " ,  ["+templateFileName+"]  ";
    }

    private void z5_closeStackTopNode() {
        log.output(automatName, 5);
        Node node = (Node) nodeStack.pop();
        Node parentNode = (Node) nodeStack.pop();
        parentNode.addNode(node);
        nodeStack.push(parentNode);
    }


    private void z6_setPositionAfterTagClosing() {
        log.output(automatName, 6);
        position = templateFileText.indexOf(">", currentTagPosition) + 1;
    }

    private void z7_saveNewElementNode() {
        log.output(automatName, 7);
        String nodeName = "";
        position = currentTagPosition + NS_START_PREFIX.length();
        while (!Character.isWhitespace(templateFileText.charAt(position)) && (">".indexOf(templateFileText.charAt(position))<0)) {
            nodeName += templateFileText.charAt(position);
            position++;
        }
        nodeStack.push(new ElementNode(nodeName));
    }

    private void z8_saveNodeAttribute() {
        log.output(automatName, 8);
        int oldPosition = position;
        String nodeString = templateFileText.substring(position, templateFileText.indexOf(">", position));
        if ((nodeString.length()>0) && (nodeString.charAt(nodeString.length() - 1) == '/'))
            nodeString = nodeString.substring(0, nodeString.length() - 1);
        int pos = 0;
        while (Character.isWhitespace(nodeString.charAt(pos))) pos++;
        String attrName = "";
        while (!Character.isWhitespace(nodeString.charAt(pos)) && ("=".indexOf(nodeString.charAt(pos))<0)) {
            attrName += nodeString.charAt(pos);
            pos++;
        }
        position = oldPosition + pos;
        while (Character.isWhitespace(nodeString.charAt(pos))) pos++;
        char ch = nodeString.charAt(pos);
        String attrValue;
        if (ch != '=') attrValue = "true";
        else {
            pos++;
            while (Character.isWhitespace(nodeString.charAt(pos))) pos++;
            ch = nodeString.charAt(pos);
            if (ch!='\"') {
                ERROR = " :    ["+templateFileName+"]    ";
                return;
            } else {
                int pos2 = nodeString.indexOf("\"", pos + 1);
                while ((pos2 + 1 < nodeString.length()) && (nodeString.charAt(pos2+1) == '"') && (pos2 > 0))
                    pos2 = nodeString.indexOf("\"", pos2 + 2);
                if (pos2 < 0) {
                    ERROR = " :    ["+templateFileName+"]  ";
                    return;
                }
                attrValue = nodeString.substring(pos + 1, pos2);
                position = oldPosition + pos2 + 1;
                pos2 = attrValue.indexOf("\"\"");
                while (pos2 >= 0) {
                    attrValue = ((pos2>0)?attrValue.substring(0, pos2):"") + attrValue.substring(pos2 + 1);
                    pos2 = attrValue.indexOf("\"\"", pos2 + 1);
                }
            }
        }
        Node node = (Node) nodeStack.pop();
        try {
            node.getAttributes().addAttribute(attrName, attrValue);
            nodeStack.push(node);
        } catch (Exception e) {
            ERROR = " : "+e.getMessage();
        }
    }

    private void z9_saveLastText() {
        log.output(automatName, 9);
        Node node = (Node) nodeStack.pop();
        node.addNode(new TextNode(templateFileText.substring(position)));
        nodeStack.push(node);
        position = templateFileText.length();
    }

    private void z10_closeAutomat() {
        log.output(automatName, 10);
        running = false;
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return "   ";
            case 1: return "  ";
            case 2: return "         ";
            case 3: return "   ";
            case 4: return "  -    ";
            case 5: return " ,    ";
            case 6: return "         ";
            case 7: return "        ";
            case 8: return "   ";
            case 9: return "   ";
            case 10: return "  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return "       ";
            case 2: return "     ";
            case 3: return "  - ";
            case 4: return "         ";
            case 5: return "   ";
            case 6: return "  ";
            case 7: return "     ";
            case 8: return "    ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return "   ";
            case 2: return " ";
            case 3: return "  ";
        }
        return "    ["+number.intValue()+"]";
    }
}
