/**
 @author Dmitriy I. Suyasov
 @since 09.09.2005 16:06:38
 */
package dsuyasov.inetDoc.functionality;

import java.util.ArrayList;

public class LogScope {

    public static final int TYPE_INPUT_VARIABLE = 0;
    public static final int TYPE_OUTPUT = 1;
    public static final int TYPE_EVENT = 2;
    public static final int TYPE_AUTOMAT_CHANGE_STATE = 3;
    public static final int TYPE_AUTOMAT_START_WORK = 4;
    public static final int TYPE_AUTOMAT_FINISH_WORK = 5;
    public static final int TYPE_AUTOMAT_WORK = 6;

    private ArrayList scope;
    private ArrayList classes;

    private ArrayList rules;

    public LogScope() {
        scope = new ArrayList();
        classes = new ArrayList();
        rules = new ArrayList();
    }

    private int getAutomatNumber(String automatName) {
        for (int iLoop = 0; iLoop < classes.size(); iLoop++)
            if (classes.get(iLoop).equals(automatName))
                return iLoop;
        classes.add(automatName);
        return classes.size() - 1;
    }

    public void addInput(String automatName, int number, boolean result) {
        scope.add(new Object[] {new Integer(TYPE_INPUT_VARIABLE), new Integer(getAutomatNumber(automatName)), new Integer(number), (result) ? Boolean.TRUE : Boolean.FALSE});
    }

    public void addOutput(String automatName, int number) {
        scope.add(new Object[] {new Integer(TYPE_OUTPUT), new Integer(getAutomatNumber(automatName)), new Integer(number)});
    }

    public void addEvent(String automatName, int number) {
        scope.add(new Object[] {new Integer(TYPE_EVENT), new Integer(getAutomatNumber(automatName)), new Integer(number)});
    }

    public void addAutomatChangeState(String automatName, int stateFrom, int stateTo) {
        scope.add(new Object[] {new Integer(TYPE_AUTOMAT_CHANGE_STATE), new Integer(getAutomatNumber(automatName)), new Integer(stateFrom), new Integer(stateTo)});
    }

    public void addAutomatStartWork(String automatName, int state) {
        scope.add(new Object[] {new Integer(TYPE_AUTOMAT_START_WORK), new Integer(getAutomatNumber(automatName)), new Integer(state)});
    }

    public void addAutomatFinishWork(String automatName, int state) {
        scope.add(new Object[] {new Integer(TYPE_AUTOMAT_FINISH_WORK), new Integer(getAutomatNumber(automatName)), new Integer(state)});
    }

    public void addAutomatWork(String automatName, boolean start) {
        scope.add(new Object[] {new Integer(TYPE_AUTOMAT_WORK), new Integer(getAutomatNumber(automatName)), (start) ? Boolean.TRUE : Boolean.FALSE});
    }

    public void addSuccessedRule(String ruleName, boolean start, String input, String regularExpression) {
        rules.add(new String[] {ruleName, (start)?"TRUE":"FALSE", input, regularExpression});
    }

    public int getRuleSize() {
        return rules.size();
    }

    public String getRuleMessage(int index) {
        String[] strings = (String[]) rules.get(index);
        return " ["+strings[0]+"]  " + (strings[1].equals("TRUE")?"":"") + "    [" + strings[2] + "].        :\n" + strings[3];
    }

    public int size() {
        return scope.size();
    }

    private Class getClass(int number) throws Exception {
        return Class.forName((String) classes.get(number));
    }

    public String getMessage(int number) {
        if ((number < 0) || (number >= size()))
            return "";
        Object[] mess = (Object[]) scope.get(number);
        try {
            int type = ((Integer) mess[0]).intValue();
            Class automatClass = getClass(((Integer) mess[1]).intValue());
            int automatNumber = automatClass.getField("AUTOMAT_NUMBER").getInt(null);
            switch (type) {
                case TYPE_INPUT_VARIABLE:
                    return "+ x" + automatNumber + "_" + ((Integer) mess[2]).intValue() + " " + automatClass.getMethod("inputVariables", new Class[] {Class.forName("java.lang.Integer")}).invoke(null, new Object[] {mess[2]})+". : "+mess[3];
                case TYPE_OUTPUT:
                    return "* z" + automatNumber + "_" + ((Integer) mess[2]).intValue() + " " + automatClass.getMethod("outputs", new Class[] {Class.forName("java.lang.Integer")}).invoke(null, new Object[] {mess[2]});
                case TYPE_EVENT:
                    return "* e" + automatNumber + "_" + ((Integer) mess[2]).intValue() + "  : " + automatClass.getMethod("events", new Class[] {Class.forName("java.lang.Integer")}).invoke(null, new Object[] {mess[2]});
                case TYPE_AUTOMAT_CHANGE_STATE:
                    return "#  A" + automatNumber + " [" + automatClass.getName().substring(automatClass.getName().lastIndexOf(".") + 1) + "]    " + ((Integer) mess[2]).intValue() + "   " + ((Integer) mess[3]).intValue() + ": " + automatClass.getMethod("states", new Class[] {Class.forName("java.lang.Integer")}).invoke(null, new Object[] {mess[3]});
                case TYPE_AUTOMAT_START_WORK:
                    return "{  A" + automatNumber + " [" + automatClass.getName().substring(automatClass.getName().lastIndexOf(".") + 1) + "]     " + ((Integer) mess[2]).intValue() + ": " + automatClass.getMethod("states", new Class[] {Class.forName("java.lang.Integer")}).invoke(null, new Object[] {mess[2]});
                case TYPE_AUTOMAT_FINISH_WORK:
                    return "}  A" + automatNumber + " [" + automatClass.getName().substring(automatClass.getName().lastIndexOf(".") + 1) + "]     " + ((Integer) mess[2]).intValue() + ": " + automatClass.getMethod("states", new Class[] {Class.forName("java.lang.Integer")}).invoke(null, new Object[] {mess[2]});
                case TYPE_AUTOMAT_WORK:
                    return "!  A" + automatNumber + " " + ((((Boolean)mess[2]).booleanValue())?"":"") + " ";
                default:
                    return " ";
            }
        } catch (Exception e) {
            return "    : "+mess;
        }
    }

}
