/**
 @author Dmitriy I. Suyasov
 @since 06.09.2005 11:14:59
    }
 */
package dsuyasov.inetDoc.functionality;

import java.util.ArrayList;
import java.util.Iterator;

public class PageDepository {

    public static final String PAGE_INDEX = "PAGE_INDEX";
    public static final String PAGES_COUNT = "PAGES_COUNT";

    private ArrayList pages;
    private int index;
    private String needToCreateFileWithName = null;
    private ArrayList globalVars;

    public PageDepository() {
        pages = new ArrayList();
        index = -1;
        needToCreateFileWithName = null;
        globalVars = new ArrayList();
    }

    public PageDepository(Page page1) {
        pages = new ArrayList();
        pages.add(page1);
        index = 0;
        needToCreateFileWithName = null;
        globalVars = new ArrayList();
    }

    public PageDepository(PageDepository pageDepository) {
        pages = new ArrayList();
        for (Iterator iter = pageDepository.pages.iterator(); iter.hasNext(); )
            pages.add(new Page((Page) iter.next()));
        index = pageDepository.index;
        needToCreateFileWithName = null;
        globalVars = new ArrayList();
    }

    public PageDepository(Page page1, String fileName) {
        pages = new ArrayList();
        pages.add(page1);
        index = 0;
        needToCreateFileWithName = fileName;
        globalVars = new ArrayList();
    }

    public PageDepository(PageDepository pageDepository, String fileName) {
        pages = new ArrayList();
        for (Iterator iter = pageDepository.pages.iterator(); iter.hasNext(); )
            pages.add(new Page((Page) iter.next()));
        index = pageDepository.index;
        needToCreateFileWithName = fileName;
        globalVars = new ArrayList();
    }

    public void addToGlobalVars(String name, String value, int level) {
        globalVars.add(new Object[] {name, value, new Integer(level)});
    }

    public String getGlobalVarName(int index) {
        return (String) ((Object[]) globalVars.get(index))[0];
    }

    public String getGlobalVarValue(int index) {
        return (String) ((Object[]) globalVars.get(index))[1];
    }

    public int getGlobalVarLevel(int index) {
        return ((Integer) ((Object[]) globalVars.get(index))[2]).intValue();
    }

    public ArrayList getGlobalVars() {
        return globalVars;
    }

    public String getFileName() {
        return needToCreateFileWithName;
    }

    public void addPage(Page page) {
        pages.add(new Page(page));
        setDepositoryVariables();
    }

    public Page getPage() {
        if (index < 0) {
            if (pages.size() > 0) {
                index = 0;
                return (Page) pages.get(index);
            } else
                return null;
        } else
            return (Page) pages.get(index);
    }

    public Page getPage(int index) {
        if ((index < 0) || (index >= pages.size()))
            return null;
        return (Page) pages.get(index);
    }

    public void setIndex(int index) {
        this.index = index;
    }

    public int getIndex() {
        return index;
    }

    public int size() {
        return pages.size();
    }

    public void remove(int index) {
        pages.remove(index);
    }

    public PageDepository replace(Page oldPage, Page newPage) {
        PageDepository pd = new PageDepository(this);
        for (int iLoop = 0; iLoop < pages.size(); iLoop++)
            if ((pages.get(iLoop).equals(oldPage))) {
                pd.remove(iLoop);
                break;
            }
        pd.addPage(newPage);
        pd.setIndex(pd.size() - 1);
        return pd;
    }

    public void setVarForAll(String variable, String value) {
        for (int iLoop = 0; iLoop < pages.size(); iLoop++) {
            Page page = (Page) pages.get(iLoop);
            for (int iLoop2 = 0; iLoop2 < page.blocSize(); iLoop2++) {
                page.getBloc(iLoop2).setVar(variable, value);
            }
        }
    }

    public void setDepositoryVariables() {
        for (int iLoop = 0; iLoop < pages.size(); iLoop++) {
            Page page = (Page) pages.get(iLoop);
            for (int iLoop2 = 0; iLoop2 < page.blocSize(); iLoop2++) {
                page.getBloc(iLoop2).setVar(PAGE_INDEX, ""+iLoop);
                page.getBloc(iLoop2).setVar(PAGES_COUNT, ""+pages.size());
            }
        }
    }

}
