/**
 @author Dmitriy I. Suyasov
 @since 01.09.2005 11:48:08
 */
package dsuyasov.inetDoc.functionality;

import dsuyasov.inetDoc.automat.Automat;

import java.util.ArrayList;

public class MassRulesProcessor extends Automat {

    public static final int AUTOMAT_NUMBER = 3;

    public static final int STATE_0_CHECK_RULES = 0;
    public static final int STATE_1_LOG_RESULT = 1;
    public static final int STATE_2_CHANGE_RULES = 2;
    public static final int STATE_3_CHECK_THE_ENDLESS_LOOP = 3;

    public static final Integer PARENT = new Integer(1);
    public static final Integer CHILD = new Integer(2);

    private RulesStarter rulesStarter;
    private RuleProcessor ruleProcessor;
    private String str;

    public ArrayList rulesSuccessed;

    public MassRulesProcessor() {
        super("dsuyasov.inetDoc.functionality.MassRulesProcessor");
    }

    public void startAutomat(RulesStarter rulesStarter) {
        this.rulesStarter = new RulesStarter(rulesStarter);
        work();
    }

    /*@
          
    */
    protected void initialize() {
        z0_initialization();
        ruleProcessor.startAutomat(rulesStarter, str);
    }

    /*@
          
        @param event   
    */
    protected void automat(int event) {
        int y_previousState = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_CHECK_RULES:
                if (x0_processorResultIsNOBODY()) {
                    z1_closeAutomat();
                } else {
                    y_state = STATE_1_LOG_RESULT;
                    log.automatChangeState(automatName, 0, 1);
                }
                break;
            case STATE_1_LOG_RESULT:
                if (x3_needToLogSuccessedRules()) {
                    z5_saveRulesSuccessWithLogger();
                    y_state = STATE_2_CHANGE_RULES;
                    log.automatChangeState(automatName, 1, 2);
                } else {
                    y_state = STATE_2_CHANGE_RULES;
                    log.automatChangeState(automatName, 1, 2);
                }
                break;
            case STATE_2_CHANGE_RULES:
                if (x1_processorResultIsPARENT()) {
                    z2_setSuccessedRuleAsParent();
                    y_state = STATE_3_CHECK_THE_ENDLESS_LOOP;
                    log.automatChangeState(automatName, 2, 3);
                } else {
                    z3_setSuccessedRuleAsChild();
                    y_state = STATE_3_CHECK_THE_ENDLESS_LOOP;
                    log.automatChangeState(automatName, 2, 3);
                }
                break;
            case STATE_3_CHECK_THE_ENDLESS_LOOP:
                if (x2_checkForEndlessLoop()) {
                    z4_setResultAsError();
                    z1_closeAutomat();
                } else {
                    y_state = STATE_0_CHECK_RULES;
                    log.automatChangeState(automatName, 3, 0);
                }
                break;
        }

        if (y_previousState != y_state)
            switch (y_state) {
                case STATE_0_CHECK_RULES:
                    ruleProcessor.startAutomat(rulesStarter, str);
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }


    //  

    private boolean x0_processorResultIsNOBODY() {
        boolean result = ruleProcessor.successedResult == RuleProcessor.NOBODY_SUCCESSED;
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_processorResultIsPARENT() {
        boolean result = ruleProcessor.successedResult == RuleProcessor.PARENT_SUCCESSED;
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_checkForEndlessLoop() {
        boolean result = false;
        for (int iLoop1 = 0; iLoop1 < rulesSuccessed.size() - 1; iLoop1++)
            for (int iLoop2 = iLoop1 + 1; iLoop2 < rulesSuccessed.size(); iLoop2++) {
                String str1 = (String) ((Object[]) rulesSuccessed.get(iLoop1))[2];
                String str2 = (String) ((Object[]) rulesSuccessed.get(iLoop2))[2];
                RulesStarter ruleStarter1 = (RulesStarter) ((Object[]) rulesSuccessed.get(iLoop1))[1];
                RulesStarter ruleStarter2 = (RulesStarter) ((Object[]) rulesSuccessed.get(iLoop2))[1];
                ArrayList id1 = (ruleStarter1.parent == null)?new ArrayList():ruleStarter1.parent.getId();
                ArrayList id2 = (ruleStarter2.parent == null)?new ArrayList():ruleStarter2.parent.getId();
                if (str1.equals(str2) && CommonVariables.getInstance().compareIds(id1,id2))
                    result = true;
            }

        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_needToLogSuccessedRules() {
        CommonVariables cv = CommonVariables.getInstance();
        boolean result = (cv.getRulesLogFile() != null) && (cv.getRulesLogFile().length() != 0);
        log.inputVariable(automatName, 3, result);
        return result;
    }

    //  

    private void z0_initialization() {
        log.output(automatName, 0);
        ruleProcessor = new RuleProcessor();
        rulesSuccessed = new ArrayList();
        str = rulesStarter.getStringStack();
    }

    private void z1_closeAutomat() {
        log.output(automatName, 1);
        running = false;
    }

    private void z2_setSuccessedRuleAsParent() {
        log.output(automatName, 2);
        str = str.substring(str.length() - ruleProcessor.successedParameter);
        rulesStarter = rulesStarter.parent.getParent();
        rulesSuccessed.add(new Object[] {PARENT, rulesStarter, str});
    }

    private void z3_setSuccessedRuleAsChild() {
        log.output(automatName, 3);
        str = rulesStarter.resultString;
        rulesStarter = ((RuleStarter) rulesStarter.rules.get(ruleProcessor.successedParameter)).getRulesStarter();
        rulesSuccessed.add(new Object[] {CHILD, rulesStarter, str});
    }

    private void z4_setResultAsError() {
        log.output(automatName, 4);
        rulesSuccessed.clear();
        rulesSuccessed.add(":   (  )");
    }

    private void z5_saveRulesSuccessWithLogger() {
        log.output(automatName, 5);
        String input = str;
        input = input.replace('\n', ' ');
        input = input.replace('\r', ' ');
        input = input.replace('\t', ' ');
        if (ruleProcessor.successedResult == RuleProcessor.PARENT_SUCCESSED) {
            log.saveTheRule(rulesStarter.parent.getName(), false, input, rulesStarter.parent.getEndHistory());
        } else {
            RulesStarter rules = ((RuleStarter) rulesStarter.rules.get(ruleProcessor.successedParameter)).getRulesStarter();
            log.saveTheRule(rules.parent.getName(), true, input, rules.parent.getStartHistory());
        }
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return "  ";
            case 2: return "    ";
            case 3: return "    ";
            case 4: return "   - ";
            case 5: return "     ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return ":  -     ";
            case 1: return ":  -    ";
            case 2: return " ";
            case 3: return "    ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return "    ";
            case 2: return " ";
            case 3: return "  ";
        }
        return "    ["+number.intValue()+"]";
    }
}
