/**
 @author Dmitriy I. Suyasov
 @since 30.08.2005 14:09:37
 */
package dsuyasov.inetDoc.functionality;

import org.w3c.dom.Node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import dsuyasov.inetDoc.automat.Automat;

public class RulesStarter extends Automat {

    public static final int AUTOMAT_NUMBER = 5;

    public static final int STATE_0_PROCESS_ALL_RULES = 0;
    public static final int STATE_1_CHOOSE_THE_RESULT = 1;

    public ArrayList rules;
    private String str;
    private ArrayList ruleResults, ruleNumbers;

    public int ruleNumber;
    public String resultString = null;
    public RuleStarter parent;
    private String stringStack = "";

    public static final String RULE_NODE_NAME = "rule";

    public RulesStarter(Node node, RuleStarter parent) throws Exception {
        super("dsuyasov.inetDoc.functionality.RulesStarter");
        rules = new ArrayList();
        this.parent = parent;

        ArrayList pre = (parent == null)?new ArrayList():parent.getId();
        byte id = 0;
        Collection ruleNodes = XMLProcessor.getNodes(node, RULE_NODE_NAME);
        for (Iterator iter = ruleNodes.iterator(); iter.hasNext(); ) {
            id++;
            ArrayList next = new ArrayList(pre);
            next.add(new Byte(id));
            rules.add(new RuleStarter((Node) iter.next(), this, next));
        }
    }

    public RulesStarter(RulesStarter rulesStarter) {
        super("dsuyasov.inetDoc.functionality.RulesStarter");
        rules = rulesStarter.rules;
        ruleNumber = rulesStarter.ruleNumber;
        resultString = rulesStarter.resultString;
        parent = rulesStarter.parent;
        stringStack = rulesStarter.getStringStack();
    }

    ////    stringStack

    public void addCharToStringStack(char character) {
        stringStack += character;
        if (parent != null)
            parent.getParent().addCharToStringStack(character);
    }

    public void clearStringStack() {
        stringStack = "";
    }

    public String getStringStack() {
        return stringStack;
    }

    public void setStringPrefix(String string) {
        stringStack = string;
    }

    ////  

    public void startAutomat() {
        str = getStringStack();
        work();
    }

    public void startAutomat(String inputString) {
        str = inputString;
        work();
    }

    /*@
          
    */
    protected void initialize() {
        z0_initializeTheResultList();
    }

    /*@
          
        @param event   
    */
    protected void automat(int event) {
        int y_previousState = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_PROCESS_ALL_RULES:
                if (!x0_allRulesProcessed()) {
                    z1_processNextRule();
                } else {
                    y_state = STATE_1_CHOOSE_THE_RESULT;
                    log.automatChangeState(automatName, 1, 2);
                }
                break;
            case STATE_1_CHOOSE_THE_RESULT:
                if (x1_allResultsChecked()) {
                    y_state = STATE_0_PROCESS_ALL_RULES;
                    log.automatChangeState(automatName, 1, 0);
                } else if (x2_currentResultIsTheBest()) {
                    z3_saveNewResult();
                    z4_killResultFromList();
                } else {
                    z4_killResultFromList();
                }
                break;
        }

        if (y_previousState != y_state)
            switch (y_state) {
                case STATE_0_PROCESS_ALL_RULES:
                    z5_closeAutomat();
                    break;
                case STATE_1_CHOOSE_THE_RESULT:
                    z2_getFinalResultsList();
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }


    //  

    private boolean x0_allRulesProcessed() {
        boolean result = ruleResults.size() >= rules.size();
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_allResultsChecked() {
        boolean result = ruleResults.size() == 0;
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_currentResultIsTheBest() {
        boolean result;
        if (ruleNumber < 0) {
            result = true;
        } else {
            Integer resultPriority = ((RuleStarter) rules.get(ruleNumber)).getPriority();
            int resultPriorityInt = (resultPriority == null)?1:resultPriority.intValue();
            Integer currentPriority = ((RuleStarter) rules.get(((Integer) ruleNumbers.get(0)).intValue())).getPriority();
            int currentPriorityInt = (currentPriority == null)?resultPriorityInt+1:((resultPriority == null)?resultPriorityInt - 1:currentPriority.intValue());
            int resultLength = resultString.length();
            int currentLength = str.length() - ((String) ruleResults.get(0)).length();

            result = (currentLength > resultLength) || ((currentLength == resultLength) && (currentPriorityInt < resultPriorityInt));
        }
        log.inputVariable(automatName, 2, result);
        return result;
    }


//  

    private void z0_initializeTheResultList() {
        log.output(automatName, 0);
        ruleResults = new ArrayList();
        resultString = str;
        ruleNumber = -1;
    }

    private void z1_processNextRule() {
        log.output(automatName, 1);
        int ruleNumber = ruleResults.size();
        RuleStarter ruleStarter = (RuleStarter) rules.get(ruleNumber);
        ruleResults.add(ruleStarter.processStart(str));
    }

    private void z2_getFinalResultsList() {
        log.output(automatName, 2);
        ArrayList newResultsList = new ArrayList();
        ruleNumbers = new ArrayList();
        for (int iLoop = 0; iLoop < ruleResults.size(); iLoop++) {
            Object next = ruleResults.get(iLoop);
            if (next instanceof String) {
                newResultsList.add(next);
                ruleNumbers.add(new Integer(iLoop));
            }
        }
        ruleResults = newResultsList;
    }

    private void z3_saveNewResult() {
        log.output(automatName, 3);
        resultString = str.substring(((String) ruleResults.get(0)).length());
        ruleNumber = ((Integer) ruleNumbers.get(0)).intValue();
    }

    private void z4_killResultFromList() {
        log.output(automatName, 4);
        ruleResults.remove(0);
        ruleNumbers.remove(0);
    }

    private void z5_closeAutomat() {
        log.output(automatName, 5);
        running = false;
    }

    public String toString() {
        String str = "";
        int index = 0;
        for (Iterator iter = rules.iterator(); iter.hasNext(); ) {
            RuleStarter ruleStarter = (RuleStarter) iter.next();
            str += "RuleStarter["+index+"] = {" + ruleStarter + "\n with children -> \n" + ruleStarter.getRulesStarter() + "}\n";
            index++;
        }
        return str;
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return "   ";
            case 1: return "  ";
            case 2: return "    ";
            case 3: return "  ";
            case 4: return "  ";
            case 5: return "  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "   ";
            case 1: return "  ";
            case 2: return "   ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return " ";
        }
        return "    ["+number.intValue()+"]";
    }

}
