/**
 @author Dmitriy Suyasov
 @since 10.10.2005 11:04:40
 */
package dsuyasov.inetDoc.aho;

import dsuyasov.inetDoc.automat.Automat;

import java.util.ArrayList;

public class AutomatBuilder {

    private static int state;
    private static ArrayList transitions;

    public static Automat build(Tree tree) throws Exception {
        tree.gotoRoot();
        state = -1;
        transitions = new ArrayList();
        BeginEndState beginEndState = buildStates(getNewState(), getNewState(), tree.getCurrentElement());
        transitionSort();
        return new AutomatInfoFinderByAho(transitions, beginEndState.getStart(), beginEndState.getEnd());
    }

    private static int getNewState() {
        state++;
        return state;
    }

    private static BeginEndState buildStates(int start, int end, TreeElement tree) throws Exception {
        String text = tree.getText();
        if (text.length() > 1) {
            if (text.charAt(text.length() - 2) != '\\') {
                char last = text.charAt(text.length() - 1);
                switch (last) {
                    case '*':
                        return buildStar(start, end,  tree);
                    case '+':
                        return buildPlus(start, end, tree);
                    case '?':
                        return buildQuestion(start, end, tree);
                    case '}':
                        return buildBraces(start, end,  tree);
                    case ')':
                        return buildBrackets(start, end, tree);
                }
            }
        }
        return buildCommonTransition(start, end, text);
    }

    private static void setNewTransition(int start, int end, String condition) {
        Transition transition = new Transition();
        transition.setCondition(condition);
        transition.setStartState(start);
        transition.setNextState(end);
        transitions.add(transition);
    }

    private static BeginEndState buildStar(int start, int end, TreeElement tree) throws Exception {
//        int start = getNewState();

        String text = tree.getText();
        text = text.substring(0, text.length() - 1);
        /*BeginEndState bes = */buildStates(start, start, new TreeElement(tree.getParent(), tree.getLinks(), text));

//        setNewTransition(start, bes.getStart(), "");
//        setNewTransition(bes.getEnd(), start, "");
        setNewTransition(start, end, "");

        BeginEndState beginEndState = new BeginEndState();
        beginEndState.setStart(start);
        beginEndState.setEnd(start);
        return beginEndState;
    }

    private static BeginEndState buildPlus(int start, int end, TreeElement tree) throws Exception {
        //int start = getNewState();
        int newState = getNewState();

        String text = tree.getText();
        text = text.substring(0, text.length() - 1);
        /*BeginEndState bes = */buildStates(start, newState, new TreeElement(tree.getParent(), tree.getLinks(), text));

//        int start = bes.getStart();
//        setNewTransition(start, bes.getStart(), "");
//        int end = bes.getEnd();
//        setNewTransition(bes.getEnd(), end, "");

        /*bes = */buildStates(newState, newState, new TreeElement(tree.getParent(), tree.getLinks(), text));

        setNewTransition(newState, end, "");
//        setNewTransition(bes.getEnd(), end, "");

        BeginEndState beginEndState = new BeginEndState();
        beginEndState.setStart(start);
        beginEndState.setEnd(end);
        return beginEndState;
    }

    private static BeginEndState buildQuestion(int start, int end, TreeElement tree) throws Exception {
//        int start = getNewState();
//        int end = getNewState();

        String text = tree.getText();
        text = text.substring(0, text.length() - 1);
        /*BeginEndState bes = */buildStates(start, end, new TreeElement(tree.getParent(), tree.getLinks(), text));

//        int start = bes.getStart();
//        int end = bes.getEnd();
//        setNewTransition(start, bes.getStart(), "");
//        setNewTransition(bes.getEnd(), end, "");
        setNewTransition(start, end, "");

        BeginEndState beginEndState = new BeginEndState();
        beginEndState.setStart(start);
        beginEndState.setEnd(end);
        return beginEndState;
    }

    private static BeginEndState buildBraces(int start, int end, TreeElement tree) throws Exception {
        String text = tree.getText();
        String period = text.substring(text.lastIndexOf('{')+1, text.lastIndexOf('}'));
        int periodStart = (period.indexOf(',') >= 0)?Integer.parseInt(period.substring(0, period.indexOf(','))):Integer.parseInt(period);
        text = text.substring(0, text.lastIndexOf('{'));

//        int start = getNewState();
        int next = start;
//        int end = getNewState();

        for (int iLoop = 0; iLoop < periodStart; iLoop++) {
            int next2 = getNewState();
            /*BeginEndState bes = */buildStates(next, next2, new TreeElement(tree.getParent(), tree.getLinks(), text));

//            int newState = getNewState();

//            setNewTransition(next, bes.getStart(), "");
//            setNewTransition(bes.getEnd(), newState, "");

            next = next2;
        }

        setNewTransition(next, end, "");

        if (period.indexOf(',') > 0) {
            int periodEnd = Integer.parseInt(period.substring(period.lastIndexOf(',') + 1));
            int difference = periodEnd - periodStart;
            for (int iLoop = 0; iLoop < difference; iLoop++) {
                int next2 = getNewState();
                /*BeginEndState bes = */buildStates(next, next2, new TreeElement(tree.getParent(), tree.getLinks(), text));

//                int newState = getNewState();

//                setNewTransition(next, bes.getStart(), "");
//                setNewTransition(bes.getEnd(), newState, "");
                setNewTransition(next2, end, "");

                next = next2;
            }
        } else {
            /*BeginEndState bes = */buildStates(next, next, new TreeElement(tree.getParent(), tree.getLinks(), text));

//            setNewTransition(next, bes.getStart(), "");
//            setNewTransition(bes.getEnd(), next, "");
        }

        BeginEndState beginEndState = new BeginEndState();
        beginEndState.setStart(start);
        beginEndState.setEnd(end);
        return beginEndState;
    }

    private static BeginEndState buildBrackets(int start, int end, TreeElement tree) throws Exception {
//        int start = getNewState();
        int next = start;
//        int end = getNewState();

        int childSize = tree.getLinkSize();
        for (int iLoop = 0; iLoop < childSize; iLoop++) {
            String nextText = (iLoop + 1 == childSize)?"|":null;
            if (nextText == null) {
                nextText = tree.getLink(iLoop + 1).getText();
                //tree.gotoParent();
            }
            if (tree.getLink(iLoop).getText().equals("|")) {
//                setNewTransition(next, end, "");
                next = start;
            } else {
                if (nextText.equals("|"))
                    /*BeginEndState bes = */buildStates(next, end, tree.getLink(iLoop));
                else {
                    int newState = getNewState();
                    buildStates(next, newState, tree.getLink(iLoop));
                    next = newState;
                }
//                setNewTransition(next, bes.getStart(), "");
//                setNewTransition(bes.getEnd(), newState, "");
//                next = bes.getEnd();
            }
//            tree.gotoParent();
        }
        if (childSize == 0) setNewTransition(next, end, "");

        BeginEndState beginEndState = new BeginEndState();
        beginEndState.setStart(start);
        beginEndState.setEnd(end);
        return beginEndState;
    }

    private static BeginEndState buildCommonTransition(int start, int end, String condition) {
//        int start = getNewState();
//        int end = getNewState();
        setNewTransition(start, end, condition);

        BeginEndState beginEndState = new BeginEndState();
        beginEndState.setStart(start);
        beginEndState.setEnd(end);
        return beginEndState;
    }

    private static void transitionSort() {
        ArrayList newTransitions = new ArrayList();
        newTransitions.add(transitions.remove(0));
        while (transitions.size() > 0) {
            int min = 0;
            for (int iLoop = 1; iLoop < transitions.size(); iLoop++) {
                if (((Transition)transitions.get(min)).getStartState() > ((Transition)transitions.get(iLoop)).getStartState())
                    min = iLoop;
            }
            newTransitions.add(transitions.remove(min));
        }
        transitions = newTransitions;
    }

    private static class BeginEndState {

        private int start;
        private int end;

        public int getStart() {
            return start;
        }

        public void setStart(int start) {
            this.start = start;
        }

        public int getEnd() {
            return end;
        }

        public void setEnd(int end) {
            this.end = end;
        }

    }

}
