/**
 @author Dmitriy I. Suyasov
 @since 24.08.2005 16:41:01
 */
package dsuyasov.inetDoc.automat;

public class AutomatLexicalRuleAnalyser extends Automat {

    public static final int AUTOMAT_NUMBER = 7;

    public static final int STATE_0_WORK_BEGINNING = 0;
    public static final int STATE_1_LEXICAL_ANALISIS = 1;
    public static final int STATE_2_READ_SYMBOL = 2;

    String regularExpression = "";
    private char symbol = ' ';
    public String lexType = "";
    public String lexValue = "";

    public AutomatLexicalRuleAnalyser() {
        super("dsuyasov.inetDoc.automat.AutomatLexicalRuleAnalyser");
    }

    public void startAutomat(String regularExpression) {
        this.regularExpression = regularExpression;
        lexType = "";
        lexValue = "";
        symbol = ' ';
        work();
    }

    public void initialize() {}

    public void automat(int event) {
        int y_statePrevious = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_WORK_BEGINNING:
                if (x0_expressionIsEmpty()) {
                    z2_setLexToEndOfWork();
                    z0_finishAutomatWork();
                } else {
                    y_state = STATE_1_LEXICAL_ANALISIS;
                    z1_takeNewSymbol();
                    log.automatChangeState(automatName, 0, 1);
                }
                break;
            case STATE_1_LEXICAL_ANALISIS:
                if (x1_symbolIsSpace()) {
                    if (x0_expressionIsEmpty()) {
                        z2_setLexToEndOfWork();
                        z0_finishAutomatWork();
                    } else {
                        z1_takeNewSymbol();
                    }
                } else if (x2_symbolIsSlasher()) {
                    if (x0_expressionIsEmpty()) {
                        y_state = STATE_0_WORK_BEGINNING;
                        z3_setLexToError();
                        log.automatChangeState(automatName, 1, 0);
                    } else {
                        y_state = STATE_2_READ_SYMBOL;
                        log.automatChangeState(automatName, 1, 2);
                    }
                } else if (x3_symbolIsDigit()) {
                    y_state = STATE_0_WORK_BEGINNING;
                    z5_setLexToDigit();
                    log.automatChangeState(automatName, 1, 0);
                } else if (x4_symbolIsSpecial()) {
                    y_state = STATE_0_WORK_BEGINNING;
                    z6_setLexToSpecialSymbol();
                    log.automatChangeState(automatName, 1, 0);
                } else {
                    y_state = STATE_0_WORK_BEGINNING;
                    z7_setLexToCommonSymbol();
                    log.automatChangeState(automatName, 1, 0);
                }
                break;
            case STATE_2_READ_SYMBOL:
                if (x1_symbolIsSpace()) {
                    if (x0_expressionIsEmpty()) {
                        y_state = STATE_0_WORK_BEGINNING;
                        z3_setLexToError();
                        log.automatChangeState(automatName, 2, 0);
                    } else {
                        z1_takeNewSymbol();
                    }
                } else {
                    y_state = STATE_0_WORK_BEGINNING;
                    z4_setLexToSlesserSymbol();
                    log.automatChangeState(automatName, 2, 0);
                }
                break;
        }

        if (y_statePrevious != y_state)
            switch (y_state) {
                case STATE_0_WORK_BEGINNING:
                    z0_finishAutomatWork();
                    break;
                case STATE_2_READ_SYMBOL:
                    z1_takeNewSymbol();
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }

    //  

    private boolean x0_expressionIsEmpty() {
        boolean result = this.regularExpression.length() == 0;
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_symbolIsSpace() {
        boolean result = symbol == ' ';
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_symbolIsSlasher() {
        boolean result = symbol == '\\';
        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_symbolIsDigit() {
        boolean result = "0123456789".indexOf(symbol) >= 0;
        log.inputVariable(automatName, 3, result);
        return result;
    }

    private boolean x4_symbolIsSpecial() {
        boolean result = "().[]+*?|{,}-^".indexOf(symbol) >= 0;
        log.inputVariable(automatName, 4, result);
        return result;
    }

    //  

    private void z0_finishAutomatWork() {
        log.output(automatName, 0);
        running = false;
    }

    private void z1_takeNewSymbol() {
        log.output(automatName, 1);
        symbol = regularExpression.charAt(0);
        regularExpression = regularExpression.substring(1);
    }

    private void z2_setLexToEndOfWork() {
        log.output(automatName, 2);
        lexType = "$";
        lexValue = "";
    }

    private void z3_setLexToError() {
        log.output(automatName, 3);
        lexType = "E";
        lexValue = "   ";
    }

    private void z4_setLexToSlesserSymbol() {
        log.output(automatName, 4);
        lexType = "\\";
        lexValue = "" + symbol;
    }

    private void z5_setLexToDigit() {
        log.output(automatName, 5);
        lexType = "D";
        lexValue = "" + symbol;
    }

    private void z6_setLexToSpecialSymbol() {
        log.output(automatName, 6);
        lexType = "" + symbol;
        lexValue = "" + symbol;
    }

    private void z7_setLexToCommonSymbol() {
        log.output(automatName, 7);
        lexType = "C";
        lexValue = "" + symbol;
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return "  ";
            case 2: return "   - $ - ";
            case 3: return "   - E - ";
            case 4: return "   - \\ ";
            case 5: return "   -    (, ), ., [, ], +, *, ?, |, {, ,, }, -, ^";
            case 6: return "   - C -  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return "  - ";
            case 2: return "  - \\";
            case 3: return "  - ";
            case 4: return "  -   (, ), ., [, ], +, *, ?, |, {, ,, }, -, ^";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return " ";
            case 2: return " ";
        }
        return "    ["+number.intValue()+"]";
    }

}
