/**
 @author Dmitriy I. Suyasov
 @since 06.09.2005 11:45:34
 */
package dsuyasov.inetDoc.automat;

import dsuyasov.inetDoc.xml.Node;
import dsuyasov.inetDoc.xml.Attributes;
import dsuyasov.inetDoc.functionality.PageDepository;
import dsuyasov.inetDoc.functionality.Page;
import dsuyasov.inetDoc.functionality.CommonVariables;
import dsuyasov.inetDoc.main.InetDocErrorConstant;

import java.util.ArrayList;
import java.util.Iterator;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.MessageFormat;

public class AutomatTemplateFiller extends Automat {

    public static final int AUTOMAT_NUMBER = 9;

    public AutomatTemplateFiller() {
        super("dsuyasov.inetDoc.automat.AutomatTemplateFiller");
    }

    public static final int STATE_0_NODE_TYPE_PROCESSING = 0;
    public static final int STATE_1_NODES_PROCESSING = 1;
    public static final int STATE_2_LOOK_THE_NODE = 2;
    public static final int STATE_3_PROCESS_COMMAND = 3;
    public static final int STATE_4_PROCESSING_CHILD_NODS = 4;
    public static final int STATE_5_FILL_SUBTEMPLATE = 5;
    public static final int STATE_6_SAVE_FILE = 6;

    public static final String WRITE_NODE_NAME = "write";
    public static final String REPLACE_NODE_NAME = "replace";
    public static final String REPLACE_ONCE_NODE_NAME = "replaceOnce";
    public static final String SAVE_NODE_NAME = "save";
    public static final String MATCH_NODE_NAME = "match";
    public static final String NOT_MATCH_NODE_NAME = "notMatch";
    public static final String EXIST_NODE_NAME = "exist";
    public static final String NOT_EXIST_NODE_NAME = "notExist";
    public static final String REPEAT_NODE_NAME = "repeat";
    public static final String SET_PAGE_NODE_NAME = "setPage";
    public static final String CREATE_FILE_NODE_NAME = "createFile";
    public static final String COMMENT_NODE_NAME = "comment";

    public static final String NAME_ATTRIBUTE = "name";
    public static final String INDEX_ATTRIBUTE = "index";
    public static final String PROPERTY_ATTRIBUTE = "property";
    public static final String VAR_ATTRIBUTE = "var";
    public static final String REPLACEWHAT_ATTRIBUTE = "replacewhat";
    public static final String REPLACETO_ATTRIBUTE = "replaceto";
    public static final String TIMES_ATTRIBUTE = "times";
    public static final String NOT_RETURN_ATTRIBUTE = "NOTRETURN";
    public static final String EXPRESSION_ATTRIBUTE = "expression";
    public static final String LEVEL_ATTRIBUTE = "level";
    public static final String PAGE_ATTRIBUTE = "page";

//    private Page page;
    private PageDepository pages;
    private ArrayList newPage;
    private Node node;

    private int nodeIndex = 0;
    private AutomatTemplateFiller automatTemplateFiller;

    public String resultText;

    public void startAutomat(Node node, PageDepository pages) {
        if ((pages == null) || (pages.size() == 0)) {
            resultText = "";
            return;
        }
        this.node = node;
        this.pages = pages;
        work();
    }

    /*@
          
    */
    protected void initialize() {
        z0_initializeAutomat();
    }

    /*@
          
        @param event   
    */
    protected void automat(int event) {
        int y_previousState = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_NODE_TYPE_PROCESSING:
                if (x0_ifNodeTypeIsTextOrAttribute()) {
                    z1_saveNodeValue();
                    y_state = STATE_6_SAVE_FILE;
                    log.automatChangeState(automatName, 0, 6);
                } else {
                    y_state = STATE_1_NODES_PROCESSING;
                    log.automatChangeState(automatName, 0, 1);
                }
                break;
            case STATE_1_NODES_PROCESSING:
                if (x1_allChildNodesProcessed()) {
                    y_state = STATE_6_SAVE_FILE;
                    log.automatChangeState(automatName, 1, 6);
                } else if (x2_childNodeIsText()) {
                    z2_saveChildNodeValue();
                    z3_increaseTheIndex();
                } else {
                    y_state = STATE_2_LOOK_THE_NODE;
                    log.automatChangeState(automatName, 1, 2);
                }
                break;
            case STATE_2_LOOK_THE_NODE:
                if (x4_childIsWrite()) {
                    z8_processWriting();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x5_childIsReplace()) {
                    z10_processReplace();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x12_childIsReplaceOnce()) {
                    z11_processReplaceOnce();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x6_childIsSave()) {
                    z16_processSave();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x7_childIsRepeat()) {
                    z9_processRepeat();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x8_childIsExist()) {
                    z12_processExist();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x9_childIsNotExist()) {
                    z13_processNotExist();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x10_childIsMatch()) {
                    z14_processMatch();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x11_childIsNotMatch()) {
                    z15_processNotMatch();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x13_childIsSetPage()) {
                    z17_processSetPage();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x14_childIsCreateFile()) {
                    z18_processCreateFile();
                    y_state = STATE_3_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 2, 3);
                } else if (x15_childIsComment()) {
                    z3_increaseTheIndex();
                    y_state = STATE_1_NODES_PROCESSING;
                    log.automatChangeState(automatName, 3, 1);
                } else {
                    z6_saveUnknownTagNameError();
                    z20_closeAutomat();
                }
                break;
            case STATE_3_PROCESS_COMMAND:
                if (x16_errorFound()) {
                    z20_closeAutomat();
                } else if (x17_nodeInsideNotEmpty()) {
                    y_state = STATE_4_PROCESSING_CHILD_NODS;
                    log.automatChangeState(automatName, 3, 4);
                } else {
                    z3_increaseTheIndex();
                    y_state = STATE_1_NODES_PROCESSING;
                    log.automatChangeState(automatName, 3, 1);
                }
                break;
            case STATE_4_PROCESSING_CHILD_NODS:
                if (x18_needToProcessOneMoreTime()) {
                    y_state = STATE_5_FILL_SUBTEMPLATE;
                    log.automatChangeState(automatName, 4, 5);
                } else {
                    z3_increaseTheIndex();
                    y_state = STATE_1_NODES_PROCESSING;
                    log.automatChangeState(automatName, 4, 1);
                }
                break;
            case STATE_5_FILL_SUBTEMPLATE:
                if (x3_automatResultIsError()) {
                    z4_saveAutomatError();
                    z20_closeAutomat();
                } else {
                    z5_saveAutomatResult();
                    z7_setCycleAsProcessed();
                    y_state = STATE_4_PROCESSING_CHILD_NODS;
                    log.automatChangeState(automatName, 5, 4);
                }
                break;
            case STATE_6_SAVE_FILE:
                if (x19_needToSaveTheTemplate()) {
                    z19_saveTheTemplate();
                    z20_closeAutomat();
                } else {
                    z20_closeAutomat();
                }
                break;
        }

        if (y_previousState != y_state)
            switch (y_state) {
                case STATE_4_PROCESSING_CHILD_NODS:
                    break;
                case STATE_5_FILL_SUBTEMPLATE:
                    automatTemplateFiller.startAutomat(node.getChildNodes().getNode(nodeIndex), (PageDepository) newPage.get(0));
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }


    //  

    private boolean x0_ifNodeTypeIsTextOrAttribute() {
        boolean result = (node.getType() == Node.ATTRIBUTE_NODE) || (node.getType() == Node.TEXT_NODE);
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_allChildNodesProcessed() {
        boolean result = nodeIndex >= node.getChildNodes().size();
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_childNodeIsText() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getType() == Node.TEXT_NODE;
        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_automatResultIsError() {
        boolean result = automatTemplateFiller.ERROR.length() > 0;
        log.inputVariable(automatName, 3, result);
        return result;
    }


    private boolean x4_childIsWrite() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(WRITE_NODE_NAME);
        log.inputVariable(automatName, 4, result);
        return result;
    }

    private boolean x5_childIsReplace() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(REPLACE_NODE_NAME);
        log.inputVariable(automatName, 5, result);
        return result;
    }

    private boolean x6_childIsSave() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(SAVE_NODE_NAME);
        log.inputVariable(automatName, 6, result);
        return result;
    }

    private boolean x7_childIsRepeat() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(REPEAT_NODE_NAME);
        log.inputVariable(automatName, 7, result);
        return result;
    }

    private boolean x8_childIsExist() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(EXIST_NODE_NAME);
        log.inputVariable(automatName, 8, result);
        return result;
    }

    private boolean x9_childIsNotExist() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(NOT_EXIST_NODE_NAME);
        log.inputVariable(automatName, 9, result);
        return result;
    }

    private boolean x10_childIsMatch() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(MATCH_NODE_NAME);
        log.inputVariable(automatName, 10, result);
        return result;
    }

    private boolean x11_childIsNotMatch() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(NOT_MATCH_NODE_NAME);
        log.inputVariable(automatName, 11, result);
        return result;
    }

    private boolean x12_childIsReplaceOnce() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(REPLACE_ONCE_NODE_NAME);
        log.inputVariable(automatName, 12, result);
        return result;
    }

    private boolean x13_childIsSetPage() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(SET_PAGE_NODE_NAME);
        log.inputVariable(automatName, 13, result);
        return result;
    }

    private boolean x14_childIsCreateFile() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(CREATE_FILE_NODE_NAME);
        log.inputVariable(automatName, 14, result);
        return result;
    }

    private boolean x15_childIsComment() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getName().equals(COMMENT_NODE_NAME);
        log.inputVariable(automatName, 15, result);
        return result;
    }

    private boolean x16_errorFound() {
        boolean result = ERROR.length() > 0;
        log.inputVariable(automatName, 16, result);
        return result;
    }

    private boolean x17_nodeInsideNotEmpty() {
        boolean result = node.getChildNodes().getNode(nodeIndex).getChildNodes().size() > 0;
        log.inputVariable(automatName, 17, result);
        return result;
    }

    private boolean x18_needToProcessOneMoreTime() {
        boolean result = newPage.size() > 0;
        log.inputVariable(automatName, 18, result);
        return result;
    }

    private boolean x19_needToSaveTheTemplate() {
        boolean result = pages.getFileName() != null;
        log.inputVariable(automatName, 18, result);
        return result;
    }

    //  

    private void z0_initializeAutomat() {
        log.output(automatName, 0);
        resultText = "";
        automatTemplateFiller = new AutomatTemplateFiller();
        nodeIndex = 0;
        ERROR = "";
    }

    private void z1_saveNodeValue() {
        log.output(automatName, 1);
        resultText += node.getValue();
    }

    private void z2_saveChildNodeValue() {
        log.output(automatName, 2);
        resultText += node.getChildNodes().getNode(nodeIndex).getValue();
    }

    private void z3_increaseTheIndex() {
        log.output(automatName, 3);
        nodeIndex++;
    }

    private void z4_saveAutomatError() {
        log.output(automatName, 4);
        ERROR = automatTemplateFiller.ERROR;
    }

    private void z5_saveAutomatResult() {
        log.output(automatName, 5);
        if ((automatTemplateFiller.pages.getFileName() == null) || (automatTemplateFiller.pages.getFileName().length() == 0))
            resultText += automatTemplateFiller.resultText;
        for (int iLoop = 0; iLoop < automatTemplateFiller.pages.getGlobalVars().size(); iLoop++) {
            pages.setVarForAll(automatTemplateFiller.pages.getGlobalVarName(iLoop), automatTemplateFiller.pages.getGlobalVarValue(iLoop));
            if (automatTemplateFiller.pages.getGlobalVarLevel(iLoop) > 1) {
                pages.addToGlobalVars(automatTemplateFiller.pages.getGlobalVarName(iLoop), automatTemplateFiller.pages.getGlobalVarValue(iLoop), automatTemplateFiller.pages.getGlobalVarLevel(iLoop) - 1);
            }
        }
    }

    private void z6_saveUnknownTagNameError() {
        log.output(automatName, 6);
        ERROR = ":    ["+node.getChildNodes().getNode(nodeIndex).getName()+"] ";
    }

    private void z7_setCycleAsProcessed() {
        log.output(automatName, 7);
        newPage.remove(0);
    }

    private void z8_processWriting() {
        log.output(automatName, 8);
        newPage = new ArrayList();
        PageDepository thisPage = new PageDepository(pages);
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);
        Node pageAttribute = attributes.getNamedAttribute(PAGE_ATTRIBUTE);

        try {
            int index;
            if (pageAttribute == null)
                index = pages.getIndex();
            else {
                try {
                    index = Integer.parseInt(pageAttribute.getValue());
                    if ((index < 0) || (index >= pages.size()))
                        throw new Exception(" [page]      -  ");
                } catch (Exception e) {
                    throw new RuntimeException(e.getMessage());
                }
            }
            if (thisNode.getAttributes().getNamedAttribute(VAR_ATTRIBUTE) != null) {
                String attribute = pages.getPage(index).getAttribute((nameAttribute==null)?null:nameAttribute.getValue(), (propertyAttribute == null)?null:propertyAttribute.getValue());
                if (thisNode.getChildNodes().size() > 0) {
                    thisPage.getPage().getBloc(thisPage.getPage().getDefault()).setVar(varAttribute.getValue(), attribute);
                    //getElement(, (varAttribute == null)?null:varAttribute.getValue());
                } else {
                    pages.getPage().getBloc(thisPage.getPage().getDefault()).setVar(varAttribute.getValue(), attribute);
                    //pages.getPage(index).getElement((nameAttribute==null)?null:nameAttribute.getValue(), (propertyAttribute == null)?null:propertyAttribute.getValue(), (varAttribute == null)?null:varAttribute.getValue());
                }
            } else
                resultText += pages.getPage(index).getElement((nameAttribute==null)?null:nameAttribute.getValue(), (propertyAttribute == null)?null:propertyAttribute.getValue(), (varAttribute == null)?null:varAttribute.getValue());
            newPage.add(thisPage);
        } catch (RuntimeException e) {
            ERROR = ": "+e.getMessage();
        }
    }

    private void z9_processRepeat() {
        log.output(automatName, 9);
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);
        Node timesAttribute = attributes.getNamedAttribute(TIMES_ATTRIBUTE);

        newPage = new ArrayList();
        try {
            ArrayList list = pages.getPage().getRepeatPages((nameAttribute==null)?null:nameAttribute.getValue(),
                                          (propertyAttribute == null)?null:propertyAttribute.getValue(),
                                          (varAttribute == null)?null:varAttribute.getValue(),
                                          (timesAttribute == null)?null:timesAttribute.getValue());
            // Save repeat index
            int postfix = 0;
            while (pages.getPage().getBloc(pages.getPage().getDefault()).getVar(Page.REPEAT_INDEX+postfix)!=null) postfix++;
            //newPage.getBloc(newPage.defaultBloc).setVar(Page.REPEAT_INDEX+postfix, ""+index);
            int index = 0;
            for (Iterator iter = list.iterator(); iter.hasNext(); ) {
                PageDepository pd = pages.replace(pages.getPage(), (Page) iter.next());
                pd.setVarForAll(Page.REPEAT_INDEX + postfix, ""+index);
                newPage.add(pd);
                index++;
            }
        } catch (RuntimeException e) {
            ERROR = ": "+e.getMessage();
        }
    }


    private void z10_processReplace() {
        log.output(automatName, 10);

        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node replaceWhat = attributes.getNamedAttribute(REPLACEWHAT_ATTRIBUTE);
        Node replaceTo = attributes.getNamedAttribute(REPLACETO_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);
        Node times = attributes.getNamedAttribute(TIMES_ATTRIBUTE);
        Node notReturn = attributes.getNamedAttribute(NOT_RETURN_ATTRIBUTE);

        newPage = new ArrayList();

        try {
            if (thisNode.getChildNodes().size() > 0) {
                PageDepository thisPage = new PageDepository(pages);
                thisPage.getPage().replace((nameAttribute==null)?null:nameAttribute.getValue(),
                                 (propertyAttribute==null)?null:propertyAttribute.getValue(),
                                 (replaceWhat==null)?null:replaceWhat.getValue(),
                                 (replaceTo==null)?null:replaceTo.getValue(),
                                 (varAttribute==null)?null:varAttribute.getValue(),
                                 (times==null)?null:times.getValue(),
                                 (notReturn==null)?null:notReturn.getValue());
                newPage.add(thisPage);
            } else {
                pages.getPage().replace((nameAttribute==null)?null:nameAttribute.getValue(),
                                 (propertyAttribute==null)?null:propertyAttribute.getValue(),
                                 (replaceWhat==null)?null:replaceWhat.getValue(),
                                 (replaceTo==null)?null:replaceTo.getValue(),
                                 (varAttribute==null)?null:varAttribute.getValue(),
                                 (times==null)?null:times.getValue(),
                                 (notReturn==null)?null:notReturn.getValue());
            }
        } catch (RuntimeException e) {
            ERROR = ": "+e.getMessage();
        }
    }

    private void z11_processReplaceOnce() {
        log.output(automatName, 11);

        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node replaceWhat = attributes.getNamedAttribute(REPLACEWHAT_ATTRIBUTE);
        Node replaceTo = attributes.getNamedAttribute(REPLACETO_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);
        Node notReturn = attributes.getNamedAttribute(NOT_RETURN_ATTRIBUTE);

        newPage = new ArrayList();

        try {
            if (thisNode.getChildNodes().size() > 0) {
                PageDepository thisPage = new PageDepository(pages);
                thisPage.getPage().replaceOnce((nameAttribute==null)?null:nameAttribute.getValue(),
                                 (propertyAttribute==null)?null:propertyAttribute.getValue(),
                                 (replaceWhat==null)?null:replaceWhat.getValue(),
                                 (replaceTo==null)?null:replaceTo.getValue(),
                                 (varAttribute==null)?null:varAttribute.getValue(),
                                 (notReturn==null)?null:notReturn.getValue());
                newPage.add(thisPage);
            } else {
                pages.getPage().replaceOnce((nameAttribute==null)?null:nameAttribute.getValue(),
                                 (propertyAttribute==null)?null:propertyAttribute.getValue(),
                                 (replaceWhat==null)?null:replaceWhat.getValue(),
                                 (replaceTo==null)?null:replaceTo.getValue(),
                                 (varAttribute==null)?null:varAttribute.getValue(),
                                 (notReturn==null)?null:notReturn.getValue());
            }
        } catch (RuntimeException e) {
            ERROR = ": "+e.getMessage();
        }
    }

    private void z12_processExist() {
        log.output(automatName, 12);

        newPage = new ArrayList();
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);
        Node pageAttribute = attributes.getNamedAttribute(PAGE_ATTRIBUTE);

        String name = pages.getPage().getPropertiesFromString((nameAttribute == null)?null:nameAttribute.getValue());
        String property = pages.getPage().getPropertiesFromString((propertyAttribute == null)?null:propertyAttribute.getValue());
        String var = pages.getPage().getPropertiesFromString((varAttribute == null)?null:varAttribute.getValue());
        String page = pages.getPage().getPropertiesFromString((pageAttribute == null)?null:pageAttribute.getValue());

        int index = 0;
        if (page == null)
            index = pages.getIndex();
        else {
            try {
                index = Integer.parseInt(page);
                if ((index < 0) || (index >= pages.size()))
                    throw new Exception(" [page]      -  ");
            } catch (Exception e) {
                ERROR = ": "+e.getMessage();
            }
        }

        if (propertyAttribute == null) {
            ERROR = ":   [exist]    [property]";
            return;
        }

        boolean exist;
        if (nameAttribute == null) {
            exist = pages.getPage(index).getBloc(pages.getPage(index).getDefault()).getVar(property) != null;
            if (!exist) exist = pages.getPage(index).getBloc(pages.getPage(index).getDefault()).getProperty(property) != null;
        } else {
            exist = pages.getPage(index).getName(name).getVar(property) != null;
            if (!exist) exist = pages.getPage(index).getName(name).getProperty(property) != null;
        }

        if (thisNode.getChildNodes().size() > 0) {
            if (exist) {
                PageDepository thisPage = new PageDepository(pages);
                if (varAttribute != null) {
                    thisPage.getPage().getBloc(thisPage.getPage().getDefault()).setVar(var, ""+exist);
                }
                newPage.add(thisPage);
            }
        } else if (varAttribute != null) {
            pages.getPage().getBloc(pages.getPage().getDefault()).setVar(var, ""+exist);
        }
    }

    private void z13_processNotExist() {
        log.output(automatName, 13);

        newPage = new ArrayList();
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);
        Node pageAttribute = attributes.getNamedAttribute(PAGE_ATTRIBUTE);

        String name = pages.getPage().getPropertiesFromString((nameAttribute == null)?null:nameAttribute.getValue());
        String property = pages.getPage().getPropertiesFromString((propertyAttribute == null)?null:propertyAttribute.getValue());
        String var = pages.getPage().getPropertiesFromString((varAttribute == null)?null:varAttribute.getValue());
        String page = pages.getPage().getPropertiesFromString((pageAttribute == null)?null:pageAttribute.getValue());

        if (propertyAttribute == null) {
            ERROR = ":   [notExist]    [property]";
            return;
        }

        int index = 0;
        if (page == null)
            index = pages.getIndex();
        else {
            try {
                index = Integer.parseInt(page);
                if ((index < 0) || (index >= pages.size()))
                    throw new Exception(" [page]      -  ");
            } catch (Exception e) {
                ERROR = ": "+e.getMessage();
            }
        }

        boolean exist;
        if (nameAttribute == null) {
            exist = pages.getPage(index).getBloc(pages.getPage(index).getDefault()).getVar(property) != null;
            if (!exist) exist = pages.getPage(index).getBloc(pages.getPage(index).getDefault()).getProperty(property) != null;
        } else {
            exist = pages.getPage(index).getName(name).getVar(property) != null;
            if (!exist) exist = pages.getPage(index).getName(name).getProperty(property) != null;
        }
        exist = !exist;

        if (thisNode.getChildNodes().size() > 0) {
            if (exist) {
                PageDepository thisPage = new PageDepository(pages);
                if (varAttribute != null) {
                    thisPage.getPage().getBloc(thisPage.getPage().getDefault()).setVar(var, ""+exist);
                }
                newPage.add(thisPage);
            }
        } else if (varAttribute != null) {
            pages.getPage().getBloc(pages.getPage().getDefault()).setVar(var, ""+exist);
        }
    }

    private void z14_processMatch() {
        log.output(automatName, 14);

        newPage = new ArrayList();
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node expressionAttribute = attributes.getNamedAttribute(EXPRESSION_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);


        try {
            boolean match = pages.getPage().match((nameAttribute==null)?null:nameAttribute.getValue(),
                                     (propertyAttribute==null)?null:propertyAttribute.getValue(),
                                     (expressionAttribute==null)?null:expressionAttribute.getValue());
            if (thisNode.getChildNodes().size() > 0) {
                if (match) {
                    PageDepository thisPage = new PageDepository(pages);
                    if (varAttribute != null)
                        thisPage.getPage().getBloc(thisPage.getPage().getDefault()).setVar(varAttribute.getValue(), ""+match);
                    newPage.add(thisPage);
                }
            } else if (varAttribute != null)
                pages.getPage().getBloc(pages.getPage().getDefault()).setVar(varAttribute.getValue(), ""+match);
        } catch (RuntimeException e) {
            ERROR = ": "+e.getMessage();
        }
    }

    private void z15_processNotMatch() {
        log.output(automatName, 15);

        newPage = new ArrayList();
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        Node propertyAttribute = attributes.getNamedAttribute(PROPERTY_ATTRIBUTE);
        Node expressionAttribute = attributes.getNamedAttribute(EXPRESSION_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);


        try {
            boolean match = pages.getPage().notMatch((nameAttribute==null)?null:nameAttribute.getValue(),
                                     (propertyAttribute==null)?null:propertyAttribute.getValue(),
                                     (expressionAttribute==null)?null:expressionAttribute.getValue());
            if (thisNode.getChildNodes().size() > 0) {
                if (match) {
                    PageDepository thisPage = new PageDepository(pages);
                    if (varAttribute != null)
                        thisPage.getPage().getBloc(thisPage.getPage().getDefault()).setVar(varAttribute.getValue(), ""+match);
                    newPage.add(thisPage);
                }
            } else if (varAttribute != null)
                pages.getPage().getBloc(pages.getPage().getDefault()).setVar(varAttribute.getValue(), ""+match);
        } catch (RuntimeException e) {
            ERROR = ": "+e.getMessage();
        }
    }

    private void z16_processSave() {
        log.output(automatName, 16);

        newPage = new ArrayList();
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node expressionAttribute = attributes.getNamedAttribute(EXPRESSION_ATTRIBUTE);
        Node varAttribute = attributes.getNamedAttribute(VAR_ATTRIBUTE);
        Node levelAttribute = attributes.getNamedAttribute(LEVEL_ATTRIBUTE);

        String expression = (expressionAttribute == null)?null:expressionAttribute.getValue();
        String var = (varAttribute == null)?null:varAttribute.getValue();
        int level = -1;
        if (levelAttribute != null) {
            try {
                level = Integer.parseInt(levelAttribute.getValue());
                if (level < 0)
                    throw new NumberFormatException();
            } catch(NumberFormatException nfe) {
                ERROR = ":  ["+LEVEL_ATTRIBUTE+"]  ["+SAVE_NODE_NAME+"]     ";
                return;
            }
        }
        try {
            String[] vne = pages.getPage().saveExpression(var, expression);
            if (thisNode.getChildNodes().size() > 0) {
                PageDepository thisPage = new PageDepository(pages);
                thisPage.setVarForAll(vne[0], vne[1]);
                newPage.add(thisPage);
                if (level >= 0)
                    pages.setVarForAll(vne[0], vne[1]);
                if (level > 0)
                    pages.addToGlobalVars(vne[0], vne[1], level);
            } else {
    //            pages.getPage().saveExpression(vne[0], vne[1]);
                pages.setVarForAll(vne[0], vne[1]);
                if (level > 0)
                    pages.addToGlobalVars(vne[0], vne[1], level);
            }
        } catch (RuntimeException e) {
            ERROR = ": " + e.getMessage();
        }

    }

    private void z17_processSetPage() {
        log.output(automatName, 17);

        newPage = new ArrayList();
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node indexAttribute = attributes.getNamedAttribute(INDEX_ATTRIBUTE);
        if (indexAttribute == null)
            ERROR = " ["+SET_PAGE_NODE_NAME+"]    ["+INDEX_ATTRIBUTE+"]   ";
        else {
            String indexText = pages.getPage().getPropertiesFromString(indexAttribute.getValue());
            try {
                int index = Integer.parseInt(indexText);
                if ((index < 0) || (index >= pages.size()))
                    throw new NumberFormatException();
                pages.setIndex(index);
            } catch(NumberFormatException nfe) {
                ERROR = " ["+INDEX_ATTRIBUTE+"]  ["+SET_PAGE_NODE_NAME+"]      -  ";
            }
        }
    }

    private void z18_processCreateFile() {
        log.output(automatName, 18);
        newPage = new ArrayList();
        Node thisNode = node.getChildNodes().getNode(nodeIndex);
        Attributes attributes = thisNode.getAttributes();

        Node nameAttribute = attributes.getNamedAttribute(NAME_ATTRIBUTE);
        if (nameAttribute == null) {
            ERROR = ":  ["+CREATE_FILE_NODE_NAME+"]    ["+NAME_ATTRIBUTE+"]";
        } else {
            String dir = CommonVariables.getInstance().getDestinationDirectory();
            dir = (dir.lastIndexOf(File.separator) == dir.length() - File.separator.length())?dir:dir+File.separator;
            String name = dir + pages.getPage().getPropertiesFromString(nameAttribute.getValue());
            if ((name != null) && (name.length() > 0)) {
                File file = new File(name);
                PageDepository thisPage = new PageDepository(pages, file.getAbsolutePath());
                newPage.add(thisPage);
            } else {
                ERROR = ":  ["+CREATE_FILE_NODE_NAME+"]     ["+NAME_ATTRIBUTE+"]";
            }
        }
    }

    private void z19_saveTheTemplate() {
        log.output(automatName, 19);

        String fileName = pages.getFileName();
        String tmpStr = fileName;
        String directoryName = "";
        while (tmpStr.indexOf(File.separator) >= 0) {
            directoryName += (directoryName.length()>0)?File.separator:"";
            directoryName += tmpStr.substring(0, tmpStr.indexOf(File.separator));
            tmpStr = tmpStr.substring(tmpStr.indexOf(File.separator) + File.separator.length());
            File dir = new File(directoryName);
            dir.mkdir();
        }
        File file = new File(fileName);

        try {
            file.createNewFile();
            FileWriter fw = new FileWriter(file);
            fw.write(resultText);
            fw.close();
        } catch (IOException ioe) {
            ERROR = MessageFormat.format(InetDocErrorConstant.ERROR_CAN_NOT_CREATE_FILE, new String[] {fileName});
        }
    }

    private void z20_closeAutomat() {
        log.output(automatName, 20);
        running = false;
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return "    ";
            case 2: return "  -  ";
            case 3: return "   -";
            case 4: return "  ";
            case 5: return "  ";
            case 6: return "  -   ";
            case 7: return "  ";
            case 8: return "  "+WRITE_NODE_NAME;
            case 9: return "  "+REPEAT_NODE_NAME;
            case 10: return "  "+REPLACE_NODE_NAME;
            case 11: return "  "+REPLACE_ONCE_NODE_NAME;
            case 12: return "  "+EXIST_NODE_NAME;
            case 13: return "  "+NOT_EXIST_NODE_NAME;
            case 14: return "  "+MATCH_NODE_NAME;
            case 15: return "  "+NOT_MATCH_NODE_NAME;
            case 16: return "  "+SAVE_NODE_NAME;
            case 17: return "  "+SET_PAGE_NODE_NAME;
            case 18: return "  "+CREATE_FILE_NODE_NAME;
            case 19: return "  ";
            case 20: return "  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "    ";
            case 1: return " -   ";
            case 2: return "-  ";
            case 3: return "-    ";
            case 4: return " - - "+WRITE_NODE_NAME;
            case 5: return " - - "+REPLACE_NODE_NAME;
            case 6: return " - - "+SAVE_NODE_NAME;
            case 7: return " - - "+REPEAT_NODE_NAME;
            case 8: return " - - "+EXIST_NODE_NAME;
            case 9: return " - - "+NOT_EXIST_NODE_NAME;
            case 10: return " - - "+MATCH_NODE_NAME;
            case 11: return " - - "+NOT_MATCH_NODE_NAME;
            case 12: return " - - "+REPLACE_ONCE_NODE_NAME;
            case 13: return " - - "+SET_PAGE_NODE_NAME;
            case 14: return " - - "+CREATE_FILE_NODE_NAME;
            case 15: return " - - "+COMMENT_NODE_NAME;
            case 16: return " ";
            case 17: return "-   ";
            case 18: return "     ";
            case 19: return "     ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return " ";
            case 2: return " ";
            case 3: return " ";
            case 4: return " -";
            case 5: return " ";
            case 6: return "  ";
        }
        return "    ["+number.intValue()+"]";
    }

}
